/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.message;

import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.PathResolver;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.ServerService;
import org.apache.dubbo.rpc.TriRpcStatus;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.BiStreamMethodHandler;
import org.apache.dubbo.rpc.stub.ServerStreamMethodHandler;
import org.apache.dubbo.rpc.stub.StubInvocationUtil;
import org.apache.dubbo.rpc.stub.StubInvoker;
import org.apache.dubbo.rpc.stub.StubMethodHandler;
import org.apache.dubbo.rpc.stub.StubSuppliers;
import org.apache.dubbo.rpc.stub.UnaryStubMethodHandler;

import com.google.protobuf.Message;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.concurrent.CompletableFuture;

public final class DubboMessageServiceTriple {

    public static final String SERVICE_NAME = MessageService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,MessageService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,MessageProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboMessageServiceTriple::newStub);
        StubSuppliers.addSupplier(MessageService.JAVA_SERVICE_NAME,  DubboMessageServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(MessageService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static MessageService newStub(Invoker<?> invoker) {
        return new MessageServiceStub((Invoker<MessageService>)invoker);
    }

    /**
         * <pre>
         *  调用终端
         * </pre>
         */
    private static final StubMethodDescriptor sendSmsCodeMethod = new StubMethodDescriptor("sendSmsCode",
    com.ywt.gapi.message.SendSmsCodeRequest.class, com.ywt.gapi.message.SendSmsCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.message.SendSmsCodeRequest::parseFrom,
    com.ywt.gapi.message.SendSmsCodeResponse::parseFrom);

    private static final StubMethodDescriptor sendSmsCodeAsyncMethod = new StubMethodDescriptor("sendSmsCode",
    com.ywt.gapi.message.SendSmsCodeRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.message.SendSmsCodeRequest::parseFrom,
    com.ywt.gapi.message.SendSmsCodeResponse::parseFrom);

    private static final StubMethodDescriptor sendSmsCodeProxyAsyncMethod = new StubMethodDescriptor("sendSmsCodeAsync",
    com.ywt.gapi.message.SendSmsCodeRequest.class, com.ywt.gapi.message.SendSmsCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.message.SendSmsCodeRequest::parseFrom,
    com.ywt.gapi.message.SendSmsCodeResponse::parseFrom);

    /**
         * <pre>
         *  调用服务名称
         * </pre>
         */
    private static final StubMethodDescriptor notifyAfterCreatePrescriptionMethod = new StubMethodDescriptor("notifyAfterCreatePrescription",
    com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor notifyAfterCreatePrescriptionAsyncMethod = new StubMethodDescriptor("notifyAfterCreatePrescription",
    com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor notifyAfterCreatePrescriptionProxyAsyncMethod = new StubMethodDescriptor("notifyAfterCreatePrescriptionAsync",
    com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);





    public static class MessageServiceStub implements MessageService{
        private final Invoker<MessageService> invoker;

        public MessageServiceStub(Invoker<MessageService> invoker) {
            this.invoker = invoker;
        }

            /**
         * <pre>
         *  调用终端
         * </pre>
         */
        @Override
        public com.ywt.gapi.message.SendSmsCodeResponse sendSmsCode(com.ywt.gapi.message.SendSmsCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, sendSmsCodeMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.message.SendSmsCodeResponse> sendSmsCodeAsync(com.ywt.gapi.message.SendSmsCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, sendSmsCodeAsyncMethod, request);
        }

            /**
         * <pre>
         *  调用终端
         * </pre>
         */
        @Override
        public void sendSmsCode(com.ywt.gapi.message.SendSmsCodeRequest request, StreamObserver<com.ywt.gapi.message.SendSmsCodeResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, sendSmsCodeMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  调用服务名称
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result notifyAfterCreatePrescription(com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest request){
            return StubInvocationUtil.unaryCall(invoker, notifyAfterCreatePrescriptionMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> notifyAfterCreatePrescriptionAsync(com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest request){
            return StubInvocationUtil.unaryCall(invoker, notifyAfterCreatePrescriptionAsyncMethod, request);
        }

            /**
         * <pre>
         *  调用服务名称
         * </pre>
         */
        @Override
        public void notifyAfterCreatePrescription(com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, notifyAfterCreatePrescriptionMethod , request, responseObserver);
        }



    }

    public static abstract class MessageServiceImplBase implements MessageService, ServerService<MessageService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<MessageService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/sendSmsCode" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/sendSmsCodeAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/notifyAfterCreatePrescription" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/notifyAfterCreatePrescriptionAsync" );

            BiConsumer<com.ywt.gapi.message.SendSmsCodeRequest, StreamObserver<com.ywt.gapi.message.SendSmsCodeResponse>> sendSmsCodeFunc = this::sendSmsCode;
            handlers.put(sendSmsCodeMethod.getMethodName(), new UnaryStubMethodHandler<>(sendSmsCodeFunc));
            BiConsumer<com.ywt.gapi.message.SendSmsCodeRequest, StreamObserver<com.ywt.gapi.message.SendSmsCodeResponse>> sendSmsCodeAsyncFunc = syncToAsync(this::sendSmsCode);
            handlers.put(sendSmsCodeProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(sendSmsCodeAsyncFunc));
            BiConsumer<com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest, StreamObserver<com.ywt.gapi.Result>> notifyAfterCreatePrescriptionFunc = this::notifyAfterCreatePrescription;
            handlers.put(notifyAfterCreatePrescriptionMethod.getMethodName(), new UnaryStubMethodHandler<>(notifyAfterCreatePrescriptionFunc));
            BiConsumer<com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest, StreamObserver<com.ywt.gapi.Result>> notifyAfterCreatePrescriptionAsyncFunc = syncToAsync(this::notifyAfterCreatePrescription);
            handlers.put(notifyAfterCreatePrescriptionProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(notifyAfterCreatePrescriptionAsyncFunc));




            return new StubInvoker<>(this, url, MessageService.class, handlers);
        }


        @Override
        public com.ywt.gapi.message.SendSmsCodeResponse sendSmsCode(com.ywt.gapi.message.SendSmsCodeRequest request){
            throw unimplementedMethodException(sendSmsCodeMethod);
        }

        @Override
        public com.ywt.gapi.Result notifyAfterCreatePrescription(com.ywt.gapi.message.NotifyAfterCreatePrescriptionRequest request){
            throw unimplementedMethodException(notifyAfterCreatePrescriptionMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
