/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.sf;

import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.PathResolver;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.ServerService;
import org.apache.dubbo.rpc.TriRpcStatus;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.BiStreamMethodHandler;
import org.apache.dubbo.rpc.stub.ServerStreamMethodHandler;
import org.apache.dubbo.rpc.stub.StubInvocationUtil;
import org.apache.dubbo.rpc.stub.StubInvoker;
import org.apache.dubbo.rpc.stub.StubMethodHandler;
import org.apache.dubbo.rpc.stub.StubSuppliers;
import org.apache.dubbo.rpc.stub.UnaryStubMethodHandler;

import com.google.protobuf.Message;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.concurrent.CompletableFuture;

public final class DubboSFServiceTriple {

    public static final String SERVICE_NAME = SFService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,SFService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,SfProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboSFServiceTriple::newStub);
        StubSuppliers.addSupplier(SFService.JAVA_SERVICE_NAME,  DubboSFServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(SFService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static SFService newStub(Invoker<?> invoker) {
        return new SFServiceStub((Invoker<SFService>)invoker);
    }

    /**
         * <pre>
         * [R]订单编号
         * </pre>
         */
    private static final StubMethodDescriptor createOrderMethod = new StubMethodDescriptor("createOrder",
    com.ywt.gapi.sf.CreateOrderRequest.class, com.ywt.gapi.sf.CreateOrderResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CreateOrderRequest::parseFrom,
    com.ywt.gapi.sf.CreateOrderResponse::parseFrom);

    private static final StubMethodDescriptor createOrderAsyncMethod = new StubMethodDescriptor("createOrder",
    com.ywt.gapi.sf.CreateOrderRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CreateOrderRequest::parseFrom,
    com.ywt.gapi.sf.CreateOrderResponse::parseFrom);

    private static final StubMethodDescriptor createOrderProxyAsyncMethod = new StubMethodDescriptor("createOrderAsync",
    com.ywt.gapi.sf.CreateOrderRequest.class, com.ywt.gapi.sf.CreateOrderResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CreateOrderRequest::parseFrom,
    com.ywt.gapi.sf.CreateOrderResponse::parseFrom);

    /**
         * <pre>
         * [O]快件产品类型。1-标准快递；2-顺丰特惠；3-电商特惠；6-即日件，12-医药专递等。详情查阅顺丰规范文档
         * </pre>
         */
    private static final StubMethodDescriptor confirmOrderMethod = new StubMethodDescriptor("confirmOrder",
    com.ywt.gapi.sf.ConfirmOrderRequest.class, com.ywt.gapi.sf.BaseResultResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.ConfirmOrderRequest::parseFrom,
    com.ywt.gapi.sf.BaseResultResponse::parseFrom);

    private static final StubMethodDescriptor confirmOrderAsyncMethod = new StubMethodDescriptor("confirmOrder",
    com.ywt.gapi.sf.ConfirmOrderRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.ConfirmOrderRequest::parseFrom,
    com.ywt.gapi.sf.BaseResultResponse::parseFrom);

    private static final StubMethodDescriptor confirmOrderProxyAsyncMethod = new StubMethodDescriptor("confirmOrderAsync",
    com.ywt.gapi.sf.ConfirmOrderRequest.class, com.ywt.gapi.sf.BaseResultResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.ConfirmOrderRequest::parseFrom,
    com.ywt.gapi.sf.BaseResultResponse::parseFrom);

    /**
         * <pre>
         * [C]寄件方地址，必须包含省市区.如果需要生成电子运单，则为必填。
         * </pre>
         */
    private static final StubMethodDescriptor checkOrderStatusMethod = new StubMethodDescriptor("checkOrderStatus",
    com.ywt.gapi.sf.CheckOrderStatusRequest.class, com.ywt.gapi.sf.CreateOrderResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CheckOrderStatusRequest::parseFrom,
    com.ywt.gapi.sf.CreateOrderResponse::parseFrom);

    private static final StubMethodDescriptor checkOrderStatusAsyncMethod = new StubMethodDescriptor("checkOrderStatus",
    com.ywt.gapi.sf.CheckOrderStatusRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CheckOrderStatusRequest::parseFrom,
    com.ywt.gapi.sf.CreateOrderResponse::parseFrom);

    private static final StubMethodDescriptor checkOrderStatusProxyAsyncMethod = new StubMethodDescriptor("checkOrderStatusAsync",
    com.ywt.gapi.sf.CheckOrderStatusRequest.class, com.ywt.gapi.sf.CreateOrderResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CheckOrderStatusRequest::parseFrom,
    com.ywt.gapi.sf.CreateOrderResponse::parseFrom);

    /**
         * <pre>
         * [R[寄件方公司
         * </pre>
         */
    private static final StubMethodDescriptor getRouteMethod = new StubMethodDescriptor("getRoute",
    com.ywt.gapi.sf.GetRouteRequest.class, com.ywt.gapi.sf.GetRouteResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.GetRouteRequest::parseFrom,
    com.ywt.gapi.sf.GetRouteResponse::parseFrom);

    private static final StubMethodDescriptor getRouteAsyncMethod = new StubMethodDescriptor("getRoute",
    com.ywt.gapi.sf.GetRouteRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.GetRouteRequest::parseFrom,
    com.ywt.gapi.sf.GetRouteResponse::parseFrom);

    private static final StubMethodDescriptor getRouteProxyAsyncMethod = new StubMethodDescriptor("getRouteAsync",
    com.ywt.gapi.sf.GetRouteRequest.class, com.ywt.gapi.sf.GetRouteResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.GetRouteRequest::parseFrom,
    com.ywt.gapi.sf.GetRouteResponse::parseFrom);

    /**
         * <pre>
         * [R[寄件方联系人
         * </pre>
         */
    private static final StubMethodDescriptor calcTimePromiseAndFreightMethod = new StubMethodDescriptor("calcTimePromiseAndFreight",
    com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest.class, com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest::parseFrom,
    com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse::parseFrom);

    private static final StubMethodDescriptor calcTimePromiseAndFreightAsyncMethod = new StubMethodDescriptor("calcTimePromiseAndFreight",
    com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest::parseFrom,
    com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse::parseFrom);

    private static final StubMethodDescriptor calcTimePromiseAndFreightProxyAsyncMethod = new StubMethodDescriptor("calcTimePromiseAndFreightAsync",
    com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest.class, com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest::parseFrom,
    com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse::parseFrom);





    public static class SFServiceStub implements SFService{
        private final Invoker<SFService> invoker;

        public SFServiceStub(Invoker<SFService> invoker) {
            this.invoker = invoker;
        }

            /**
         * <pre>
         * [R]订单编号
         * </pre>
         */
        @Override
        public com.ywt.gapi.sf.CreateOrderResponse createOrder(com.ywt.gapi.sf.CreateOrderRequest request){
            return StubInvocationUtil.unaryCall(invoker, createOrderMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.sf.CreateOrderResponse> createOrderAsync(com.ywt.gapi.sf.CreateOrderRequest request){
            return StubInvocationUtil.unaryCall(invoker, createOrderAsyncMethod, request);
        }

            /**
         * <pre>
         * [R]订单编号
         * </pre>
         */
        @Override
        public void createOrder(com.ywt.gapi.sf.CreateOrderRequest request, StreamObserver<com.ywt.gapi.sf.CreateOrderResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, createOrderMethod , request, responseObserver);
        }
            /**
         * <pre>
         * [O]快件产品类型。1-标准快递；2-顺丰特惠；3-电商特惠；6-即日件，12-医药专递等。详情查阅顺丰规范文档
         * </pre>
         */
        @Override
        public com.ywt.gapi.sf.BaseResultResponse confirmOrder(com.ywt.gapi.sf.ConfirmOrderRequest request){
            return StubInvocationUtil.unaryCall(invoker, confirmOrderMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.sf.BaseResultResponse> confirmOrderAsync(com.ywt.gapi.sf.ConfirmOrderRequest request){
            return StubInvocationUtil.unaryCall(invoker, confirmOrderAsyncMethod, request);
        }

            /**
         * <pre>
         * [O]快件产品类型。1-标准快递；2-顺丰特惠；3-电商特惠；6-即日件，12-医药专递等。详情查阅顺丰规范文档
         * </pre>
         */
        @Override
        public void confirmOrder(com.ywt.gapi.sf.ConfirmOrderRequest request, StreamObserver<com.ywt.gapi.sf.BaseResultResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, confirmOrderMethod , request, responseObserver);
        }
            /**
         * <pre>
         * [C]寄件方地址，必须包含省市区.如果需要生成电子运单，则为必填。
         * </pre>
         */
        @Override
        public com.ywt.gapi.sf.CreateOrderResponse checkOrderStatus(com.ywt.gapi.sf.CheckOrderStatusRequest request){
            return StubInvocationUtil.unaryCall(invoker, checkOrderStatusMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.sf.CreateOrderResponse> checkOrderStatusAsync(com.ywt.gapi.sf.CheckOrderStatusRequest request){
            return StubInvocationUtil.unaryCall(invoker, checkOrderStatusAsyncMethod, request);
        }

            /**
         * <pre>
         * [C]寄件方地址，必须包含省市区.如果需要生成电子运单，则为必填。
         * </pre>
         */
        @Override
        public void checkOrderStatus(com.ywt.gapi.sf.CheckOrderStatusRequest request, StreamObserver<com.ywt.gapi.sf.CreateOrderResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, checkOrderStatusMethod , request, responseObserver);
        }
            /**
         * <pre>
         * [R[寄件方公司
         * </pre>
         */
        @Override
        public com.ywt.gapi.sf.GetRouteResponse getRoute(com.ywt.gapi.sf.GetRouteRequest request){
            return StubInvocationUtil.unaryCall(invoker, getRouteMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.sf.GetRouteResponse> getRouteAsync(com.ywt.gapi.sf.GetRouteRequest request){
            return StubInvocationUtil.unaryCall(invoker, getRouteAsyncMethod, request);
        }

            /**
         * <pre>
         * [R[寄件方公司
         * </pre>
         */
        @Override
        public void getRoute(com.ywt.gapi.sf.GetRouteRequest request, StreamObserver<com.ywt.gapi.sf.GetRouteResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getRouteMethod , request, responseObserver);
        }
            /**
         * <pre>
         * [R[寄件方联系人
         * </pre>
         */
        @Override
        public com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse calcTimePromiseAndFreight(com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest request){
            return StubInvocationUtil.unaryCall(invoker, calcTimePromiseAndFreightMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse> calcTimePromiseAndFreightAsync(com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest request){
            return StubInvocationUtil.unaryCall(invoker, calcTimePromiseAndFreightAsyncMethod, request);
        }

            /**
         * <pre>
         * [R[寄件方联系人
         * </pre>
         */
        @Override
        public void calcTimePromiseAndFreight(com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest request, StreamObserver<com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, calcTimePromiseAndFreightMethod , request, responseObserver);
        }



    }

    public static abstract class SFServiceImplBase implements SFService, ServerService<SFService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<SFService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/createOrder" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/createOrderAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/confirmOrder" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/confirmOrderAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/checkOrderStatus" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/checkOrderStatusAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getRoute" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getRouteAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/calcTimePromiseAndFreight" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/calcTimePromiseAndFreightAsync" );

            BiConsumer<com.ywt.gapi.sf.CreateOrderRequest, StreamObserver<com.ywt.gapi.sf.CreateOrderResponse>> createOrderFunc = this::createOrder;
            handlers.put(createOrderMethod.getMethodName(), new UnaryStubMethodHandler<>(createOrderFunc));
            BiConsumer<com.ywt.gapi.sf.CreateOrderRequest, StreamObserver<com.ywt.gapi.sf.CreateOrderResponse>> createOrderAsyncFunc = syncToAsync(this::createOrder);
            handlers.put(createOrderProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(createOrderAsyncFunc));
            BiConsumer<com.ywt.gapi.sf.ConfirmOrderRequest, StreamObserver<com.ywt.gapi.sf.BaseResultResponse>> confirmOrderFunc = this::confirmOrder;
            handlers.put(confirmOrderMethod.getMethodName(), new UnaryStubMethodHandler<>(confirmOrderFunc));
            BiConsumer<com.ywt.gapi.sf.ConfirmOrderRequest, StreamObserver<com.ywt.gapi.sf.BaseResultResponse>> confirmOrderAsyncFunc = syncToAsync(this::confirmOrder);
            handlers.put(confirmOrderProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(confirmOrderAsyncFunc));
            BiConsumer<com.ywt.gapi.sf.CheckOrderStatusRequest, StreamObserver<com.ywt.gapi.sf.CreateOrderResponse>> checkOrderStatusFunc = this::checkOrderStatus;
            handlers.put(checkOrderStatusMethod.getMethodName(), new UnaryStubMethodHandler<>(checkOrderStatusFunc));
            BiConsumer<com.ywt.gapi.sf.CheckOrderStatusRequest, StreamObserver<com.ywt.gapi.sf.CreateOrderResponse>> checkOrderStatusAsyncFunc = syncToAsync(this::checkOrderStatus);
            handlers.put(checkOrderStatusProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(checkOrderStatusAsyncFunc));
            BiConsumer<com.ywt.gapi.sf.GetRouteRequest, StreamObserver<com.ywt.gapi.sf.GetRouteResponse>> getRouteFunc = this::getRoute;
            handlers.put(getRouteMethod.getMethodName(), new UnaryStubMethodHandler<>(getRouteFunc));
            BiConsumer<com.ywt.gapi.sf.GetRouteRequest, StreamObserver<com.ywt.gapi.sf.GetRouteResponse>> getRouteAsyncFunc = syncToAsync(this::getRoute);
            handlers.put(getRouteProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getRouteAsyncFunc));
            BiConsumer<com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest, StreamObserver<com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse>> calcTimePromiseAndFreightFunc = this::calcTimePromiseAndFreight;
            handlers.put(calcTimePromiseAndFreightMethod.getMethodName(), new UnaryStubMethodHandler<>(calcTimePromiseAndFreightFunc));
            BiConsumer<com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest, StreamObserver<com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse>> calcTimePromiseAndFreightAsyncFunc = syncToAsync(this::calcTimePromiseAndFreight);
            handlers.put(calcTimePromiseAndFreightProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(calcTimePromiseAndFreightAsyncFunc));




            return new StubInvoker<>(this, url, SFService.class, handlers);
        }


        @Override
        public com.ywt.gapi.sf.CreateOrderResponse createOrder(com.ywt.gapi.sf.CreateOrderRequest request){
            throw unimplementedMethodException(createOrderMethod);
        }

        @Override
        public com.ywt.gapi.sf.BaseResultResponse confirmOrder(com.ywt.gapi.sf.ConfirmOrderRequest request){
            throw unimplementedMethodException(confirmOrderMethod);
        }

        @Override
        public com.ywt.gapi.sf.CreateOrderResponse checkOrderStatus(com.ywt.gapi.sf.CheckOrderStatusRequest request){
            throw unimplementedMethodException(checkOrderStatusMethod);
        }

        @Override
        public com.ywt.gapi.sf.GetRouteResponse getRoute(com.ywt.gapi.sf.GetRouteRequest request){
            throw unimplementedMethodException(getRouteMethod);
        }

        @Override
        public com.ywt.gapi.sf.CalcTimePromiseAndFreightResponse calcTimePromiseAndFreight(com.ywt.gapi.sf.CalcTimePromiseAndFreightRequest request){
            throw unimplementedMethodException(calcTimePromiseAndFreightMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
