/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.system;

import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.PathResolver;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.ServerService;
import org.apache.dubbo.rpc.TriRpcStatus;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.BiStreamMethodHandler;
import org.apache.dubbo.rpc.stub.ServerStreamMethodHandler;
import org.apache.dubbo.rpc.stub.StubInvocationUtil;
import org.apache.dubbo.rpc.stub.StubInvoker;
import org.apache.dubbo.rpc.stub.StubMethodHandler;
import org.apache.dubbo.rpc.stub.StubSuppliers;
import org.apache.dubbo.rpc.stub.UnaryStubMethodHandler;

import com.google.protobuf.Message;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.concurrent.CompletableFuture;

public final class DubboSystemServiceTriple {

    public static final String SERVICE_NAME = SystemService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,SystemService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,SystemProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboSystemServiceTriple::newStub);
        StubSuppliers.addSupplier(SystemService.JAVA_SERVICE_NAME,  DubboSystemServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(SystemService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static SystemService newStub(Invoker<?> invoker) {
        return new SystemServiceStub((Invoker<SystemService>)invoker);
    }

    /**
         * <pre>
         * 医院Id
         * </pre>
         */
    private static final StubMethodDescriptor getHospitalListMethod = new StubMethodDescriptor("getHospitalList",
    com.ywt.gapi.system.HospitalListRequest.class, com.ywt.gapi.system.HospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalListAsyncMethod = new StubMethodDescriptor("getHospitalList",
    com.ywt.gapi.system.HospitalListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalListProxyAsyncMethod = new StubMethodDescriptor("getHospitalListAsync",
    com.ywt.gapi.system.HospitalListRequest.class, com.ywt.gapi.system.HospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    /**
         * <pre>
         * 医院名称
         * </pre>
         */
    private static final StubMethodDescriptor getDeptListMethod = new StubMethodDescriptor("getDeptList",
    com.ywt.gapi.system.DeptListRequest.class, com.ywt.gapi.system.DeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.DeptListRequest::parseFrom,
    com.ywt.gapi.system.DeptListResponse::parseFrom);

    private static final StubMethodDescriptor getDeptListAsyncMethod = new StubMethodDescriptor("getDeptList",
    com.ywt.gapi.system.DeptListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.DeptListRequest::parseFrom,
    com.ywt.gapi.system.DeptListResponse::parseFrom);

    private static final StubMethodDescriptor getDeptListProxyAsyncMethod = new StubMethodDescriptor("getDeptListAsync",
    com.ywt.gapi.system.DeptListRequest.class, com.ywt.gapi.system.DeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.DeptListRequest::parseFrom,
    com.ywt.gapi.system.DeptListResponse::parseFrom);

    /**
         * <pre>
         * 地区Id
         * </pre>
         */
    private static final StubMethodDescriptor getHospitalDeptListMethod = new StubMethodDescriptor("getHospitalDeptList",
    com.ywt.gapi.system.HospitalDeptListRequest.class, com.ywt.gapi.system.HospitalDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalDeptListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalDeptListAsyncMethod = new StubMethodDescriptor("getHospitalDeptList",
    com.ywt.gapi.system.HospitalDeptListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalDeptListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalDeptListProxyAsyncMethod = new StubMethodDescriptor("getHospitalDeptListAsync",
    com.ywt.gapi.system.HospitalDeptListRequest.class, com.ywt.gapi.system.HospitalDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalDeptListResponse::parseFrom);

    /**
         * <pre>
         * 医院等级
         * </pre>
         */
    private static final StubMethodDescriptor checkAppVersionMethod = new StubMethodDescriptor("checkAppVersion",
    com.ywt.gapi.system.CheckAppVersionRequest.class, com.ywt.gapi.system.CheckAppVersionResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CheckAppVersionRequest::parseFrom,
    com.ywt.gapi.system.CheckAppVersionResponse::parseFrom);

    private static final StubMethodDescriptor checkAppVersionAsyncMethod = new StubMethodDescriptor("checkAppVersion",
    com.ywt.gapi.system.CheckAppVersionRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CheckAppVersionRequest::parseFrom,
    com.ywt.gapi.system.CheckAppVersionResponse::parseFrom);

    private static final StubMethodDescriptor checkAppVersionProxyAsyncMethod = new StubMethodDescriptor("checkAppVersionAsync",
    com.ywt.gapi.system.CheckAppVersionRequest.class, com.ywt.gapi.system.CheckAppVersionResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CheckAppVersionRequest::parseFrom,
    com.ywt.gapi.system.CheckAppVersionResponse::parseFrom);

    /**
         * <pre>
         * 医院Logo
         * </pre>
         */
    private static final StubMethodDescriptor getAreaListMethod = new StubMethodDescriptor("getAreaList",
    com.ywt.gapi.system.GetAreaListRequest.class, com.ywt.gapi.system.GetAreaListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaListResponse::parseFrom);

    private static final StubMethodDescriptor getAreaListAsyncMethod = new StubMethodDescriptor("getAreaList",
    com.ywt.gapi.system.GetAreaListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaListResponse::parseFrom);

    private static final StubMethodDescriptor getAreaListProxyAsyncMethod = new StubMethodDescriptor("getAreaListAsync",
    com.ywt.gapi.system.GetAreaListRequest.class, com.ywt.gapi.system.GetAreaListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaListResponse::parseFrom);

    /**
         * <pre>
         * 详细地址
         * </pre>
         */
    private static final StubMethodDescriptor getAddressListMethod = new StubMethodDescriptor("getAddressList",
    com.ywt.gapi.system.GetAddressListRequest.class, com.ywt.gapi.system.GetAddressListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressListRequest::parseFrom,
    com.ywt.gapi.system.GetAddressListResponse::parseFrom);

    private static final StubMethodDescriptor getAddressListAsyncMethod = new StubMethodDescriptor("getAddressList",
    com.ywt.gapi.system.GetAddressListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressListRequest::parseFrom,
    com.ywt.gapi.system.GetAddressListResponse::parseFrom);

    private static final StubMethodDescriptor getAddressListProxyAsyncMethod = new StubMethodDescriptor("getAddressListAsync",
    com.ywt.gapi.system.GetAddressListRequest.class, com.ywt.gapi.system.GetAddressListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressListRequest::parseFrom,
    com.ywt.gapi.system.GetAddressListResponse::parseFrom);

    /**
         * <pre>
         * 能否挂号
         * </pre>
         */
    private static final StubMethodDescriptor saveAddressMethod = new StubMethodDescriptor("saveAddress",
    com.ywt.gapi.system.SaveAddressRequest.class, com.ywt.gapi.system.SaveAddressResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAddressRequest::parseFrom,
    com.ywt.gapi.system.SaveAddressResponse::parseFrom);

    private static final StubMethodDescriptor saveAddressAsyncMethod = new StubMethodDescriptor("saveAddress",
    com.ywt.gapi.system.SaveAddressRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAddressRequest::parseFrom,
    com.ywt.gapi.system.SaveAddressResponse::parseFrom);

    private static final StubMethodDescriptor saveAddressProxyAsyncMethod = new StubMethodDescriptor("saveAddressAsync",
    com.ywt.gapi.system.SaveAddressRequest.class, com.ywt.gapi.system.SaveAddressResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAddressRequest::parseFrom,
    com.ywt.gapi.system.SaveAddressResponse::parseFrom);

    /**
         * <pre>
         * 状态(暂用于表示用户是否关注)
         * </pre>
         */
    private static final StubMethodDescriptor deleteAddressMethod = new StubMethodDescriptor("deleteAddress",
    com.ywt.gapi.system.DeleteAddressRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.DeleteAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor deleteAddressAsyncMethod = new StubMethodDescriptor("deleteAddress",
    com.ywt.gapi.system.DeleteAddressRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.DeleteAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor deleteAddressProxyAsyncMethod = new StubMethodDescriptor("deleteAddressAsync",
    com.ywt.gapi.system.DeleteAddressRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.DeleteAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 父ID
         * </pre>
         */
    private static final StubMethodDescriptor getAddressMethod = new StubMethodDescriptor("getAddress",
    com.ywt.gapi.system.GetAddressRequest.class, com.ywt.gapi.system.GetAddressResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressRequest::parseFrom,
    com.ywt.gapi.system.GetAddressResponse::parseFrom);

    private static final StubMethodDescriptor getAddressAsyncMethod = new StubMethodDescriptor("getAddress",
    com.ywt.gapi.system.GetAddressRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressRequest::parseFrom,
    com.ywt.gapi.system.GetAddressResponse::parseFrom);

    private static final StubMethodDescriptor getAddressProxyAsyncMethod = new StubMethodDescriptor("getAddressAsync",
    com.ywt.gapi.system.GetAddressRequest.class, com.ywt.gapi.system.GetAddressResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressRequest::parseFrom,
    com.ywt.gapi.system.GetAddressResponse::parseFrom);

    /**
         * <pre>
         * 状态
         * </pre>
         */
    private static final StubMethodDescriptor getAreaTreeNodeListMethod = new StubMethodDescriptor("getAreaTreeNodeList",
    com.ywt.gapi.system.GetAreaTreeNodeListRequest.class, com.ywt.gapi.system.GetAreaTreeNodeListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaTreeNodeListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaTreeNodeListResponse::parseFrom);

    private static final StubMethodDescriptor getAreaTreeNodeListAsyncMethod = new StubMethodDescriptor("getAreaTreeNodeList",
    com.ywt.gapi.system.GetAreaTreeNodeListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaTreeNodeListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaTreeNodeListResponse::parseFrom);

    private static final StubMethodDescriptor getAreaTreeNodeListProxyAsyncMethod = new StubMethodDescriptor("getAreaTreeNodeListAsync",
    com.ywt.gapi.system.GetAreaTreeNodeListRequest.class, com.ywt.gapi.system.GetAreaTreeNodeListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaTreeNodeListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaTreeNodeListResponse::parseFrom);

    /**
         * <pre>
         * 添加时间
         * </pre>
         */
    private static final StubMethodDescriptor getAppAreaListMethod = new StubMethodDescriptor("getAppAreaList",
    com.ywt.gapi.system.AreaListRequest.class, com.ywt.gapi.system.AreaListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.AreaListRequest::parseFrom,
    com.ywt.gapi.system.AreaListResponse::parseFrom);

    private static final StubMethodDescriptor getAppAreaListAsyncMethod = new StubMethodDescriptor("getAppAreaList",
    com.ywt.gapi.system.AreaListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.AreaListRequest::parseFrom,
    com.ywt.gapi.system.AreaListResponse::parseFrom);

    private static final StubMethodDescriptor getAppAreaListProxyAsyncMethod = new StubMethodDescriptor("getAppAreaListAsync",
    com.ywt.gapi.system.AreaListRequest.class, com.ywt.gapi.system.AreaListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.AreaListRequest::parseFrom,
    com.ywt.gapi.system.AreaListResponse::parseFrom);

    /**
         * <pre>
         * 修改时间
         * </pre>
         */
    private static final StubMethodDescriptor getAreaHospitalListMethod = new StubMethodDescriptor("getAreaHospitalList",
    com.ywt.gapi.system.AreaHospitalListRequest.class, com.ywt.gapi.system.AreaHospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.AreaHospitalListRequest::parseFrom,
    com.ywt.gapi.system.AreaHospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getAreaHospitalListAsyncMethod = new StubMethodDescriptor("getAreaHospitalList",
    com.ywt.gapi.system.AreaHospitalListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.AreaHospitalListRequest::parseFrom,
    com.ywt.gapi.system.AreaHospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getAreaHospitalListProxyAsyncMethod = new StubMethodDescriptor("getAreaHospitalListAsync",
    com.ywt.gapi.system.AreaHospitalListRequest.class, com.ywt.gapi.system.AreaHospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.AreaHospitalListRequest::parseFrom,
    com.ywt.gapi.system.AreaHospitalListResponse::parseFrom);

    /**
         * <pre>
         * 省Id
         * </pre>
         */
    private static final StubMethodDescriptor getWeChatConfigMethod = new StubMethodDescriptor("getWeChatConfig",
    com.ywt.gapi.system.GetWeChatConfigRequest.class, com.ywt.gapi.system.GetWeChatConfigResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetWeChatConfigRequest::parseFrom,
    com.ywt.gapi.system.GetWeChatConfigResponse::parseFrom);

    private static final StubMethodDescriptor getWeChatConfigAsyncMethod = new StubMethodDescriptor("getWeChatConfig",
    com.ywt.gapi.system.GetWeChatConfigRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetWeChatConfigRequest::parseFrom,
    com.ywt.gapi.system.GetWeChatConfigResponse::parseFrom);

    private static final StubMethodDescriptor getWeChatConfigProxyAsyncMethod = new StubMethodDescriptor("getWeChatConfigAsync",
    com.ywt.gapi.system.GetWeChatConfigRequest.class, com.ywt.gapi.system.GetWeChatConfigResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetWeChatConfigRequest::parseFrom,
    com.ywt.gapi.system.GetWeChatConfigResponse::parseFrom);

    /**
         * <pre>
         * 市Id
         * </pre>
         */
    private static final StubMethodDescriptor getDeptByYiDaCodeMethod = new StubMethodDescriptor("getDeptByYiDaCode",
    com.ywt.gapi.system.GetDeptByYiDaCodeRequest.class, com.ywt.gapi.system.GetDeptByYiDaCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDeptByYiDaCodeRequest::parseFrom,
    com.ywt.gapi.system.GetDeptByYiDaCodeResponse::parseFrom);

    private static final StubMethodDescriptor getDeptByYiDaCodeAsyncMethod = new StubMethodDescriptor("getDeptByYiDaCode",
    com.ywt.gapi.system.GetDeptByYiDaCodeRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDeptByYiDaCodeRequest::parseFrom,
    com.ywt.gapi.system.GetDeptByYiDaCodeResponse::parseFrom);

    private static final StubMethodDescriptor getDeptByYiDaCodeProxyAsyncMethod = new StubMethodDescriptor("getDeptByYiDaCodeAsync",
    com.ywt.gapi.system.GetDeptByYiDaCodeRequest.class, com.ywt.gapi.system.GetDeptByYiDaCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDeptByYiDaCodeRequest::parseFrom,
    com.ywt.gapi.system.GetDeptByYiDaCodeResponse::parseFrom);

    /**
         * <pre>
         * 区Id
         * </pre>
         */
    private static final StubMethodDescriptor getYiDaDoctorCodeMethod = new StubMethodDescriptor("getYiDaDoctorCode",
    com.ywt.gapi.system.GetYiDaDoctorCodeRequest.class, com.ywt.gapi.system.GetYiDaDoctorCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetYiDaDoctorCodeRequest::parseFrom,
    com.ywt.gapi.system.GetYiDaDoctorCodeResponse::parseFrom);

    private static final StubMethodDescriptor getYiDaDoctorCodeAsyncMethod = new StubMethodDescriptor("getYiDaDoctorCode",
    com.ywt.gapi.system.GetYiDaDoctorCodeRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetYiDaDoctorCodeRequest::parseFrom,
    com.ywt.gapi.system.GetYiDaDoctorCodeResponse::parseFrom);

    private static final StubMethodDescriptor getYiDaDoctorCodeProxyAsyncMethod = new StubMethodDescriptor("getYiDaDoctorCodeAsync",
    com.ywt.gapi.system.GetYiDaDoctorCodeRequest.class, com.ywt.gapi.system.GetYiDaDoctorCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetYiDaDoctorCodeRequest::parseFrom,
    com.ywt.gapi.system.GetYiDaDoctorCodeResponse::parseFrom);

    /**
         * <pre>
         * 获取国际疾病分类
         * </pre>
         */
    private static final StubMethodDescriptor getIcdCodeListMethod = new StubMethodDescriptor("getIcdCodeList",
    com.ywt.gapi.system.GetIcdCodeListRequest.class, com.ywt.gapi.system.GetIcdCodeListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetIcdCodeListRequest::parseFrom,
    com.ywt.gapi.system.GetIcdCodeListResponse::parseFrom);

    private static final StubMethodDescriptor getIcdCodeListAsyncMethod = new StubMethodDescriptor("getIcdCodeList",
    com.ywt.gapi.system.GetIcdCodeListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetIcdCodeListRequest::parseFrom,
    com.ywt.gapi.system.GetIcdCodeListResponse::parseFrom);

    private static final StubMethodDescriptor getIcdCodeListProxyAsyncMethod = new StubMethodDescriptor("getIcdCodeListAsync",
    com.ywt.gapi.system.GetIcdCodeListRequest.class, com.ywt.gapi.system.GetIcdCodeListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetIcdCodeListRequest::parseFrom,
    com.ywt.gapi.system.GetIcdCodeListResponse::parseFrom);

    /**
         * <pre>
         * 得到所有地区集合，包括区、县
         * </pre>
         */
    private static final StubMethodDescriptor getAllAreaListMethod = new StubMethodDescriptor("getAllAreaList",
    com.ywt.gapi.system.GetAreaListRequest.class, com.ywt.gapi.system.GetAreaListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaListResponse::parseFrom);

    private static final StubMethodDescriptor getAllAreaListAsyncMethod = new StubMethodDescriptor("getAllAreaList",
    com.ywt.gapi.system.GetAreaListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaListResponse::parseFrom);

    private static final StubMethodDescriptor getAllAreaListProxyAsyncMethod = new StubMethodDescriptor("getAllAreaListAsync",
    com.ywt.gapi.system.GetAreaListRequest.class, com.ywt.gapi.system.GetAreaListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAreaListRequest::parseFrom,
    com.ywt.gapi.system.GetAreaListResponse::parseFrom);

    /**
         * <pre>
         * 获取医生地址列表 create by daiyihua
         * </pre>
         */
    private static final StubMethodDescriptor getDoctorAddressListMethod = new StubMethodDescriptor("getDoctorAddressList",
    com.ywt.gapi.system.GetAddressListRequest.class, com.ywt.gapi.system.GetAddressListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressListRequest::parseFrom,
    com.ywt.gapi.system.GetAddressListResponse::parseFrom);

    private static final StubMethodDescriptor getDoctorAddressListAsyncMethod = new StubMethodDescriptor("getDoctorAddressList",
    com.ywt.gapi.system.GetAddressListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressListRequest::parseFrom,
    com.ywt.gapi.system.GetAddressListResponse::parseFrom);

    private static final StubMethodDescriptor getDoctorAddressListProxyAsyncMethod = new StubMethodDescriptor("getDoctorAddressListAsync",
    com.ywt.gapi.system.GetAddressListRequest.class, com.ywt.gapi.system.GetAddressListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetAddressListRequest::parseFrom,
    com.ywt.gapi.system.GetAddressListResponse::parseFrom);

    /**
         * <pre>
         * 医生保存地址 create by daiyihua
         * </pre>
         */
    private static final StubMethodDescriptor doctorSaveAddressMethod = new StubMethodDescriptor("doctorSaveAddress",
    com.ywt.gapi.system.SaveAddressRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor doctorSaveAddressAsyncMethod = new StubMethodDescriptor("doctorSaveAddress",
    com.ywt.gapi.system.SaveAddressRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor doctorSaveAddressProxyAsyncMethod = new StubMethodDescriptor("doctorSaveAddressAsync",
    com.ywt.gapi.system.SaveAddressRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 医生设置默认地址 create by daiyihua
         * </pre>
         */
    private static final StubMethodDescriptor settingDefaultAddressMethod = new StubMethodDescriptor("settingDefaultAddress",
    com.ywt.gapi.system.SettingDefaultAddressRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SettingDefaultAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor settingDefaultAddressAsyncMethod = new StubMethodDescriptor("settingDefaultAddress",
    com.ywt.gapi.system.SettingDefaultAddressRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SettingDefaultAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor settingDefaultAddressProxyAsyncMethod = new StubMethodDescriptor("settingDefaultAddressAsync",
    com.ywt.gapi.system.SettingDefaultAddressRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SettingDefaultAddressRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 获取消息模板列表
         * </pre>
         */
    private static final StubMethodDescriptor getMsgTplListByModuleMethod = new StubMethodDescriptor("getMsgTplListByModule",
    com.ywt.gapi.system.GetMsgTplListByModuleRequest.class, com.ywt.gapi.system.GetMsgTplListByModuleResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetMsgTplListByModuleRequest::parseFrom,
    com.ywt.gapi.system.GetMsgTplListByModuleResponse::parseFrom);

    private static final StubMethodDescriptor getMsgTplListByModuleAsyncMethod = new StubMethodDescriptor("getMsgTplListByModule",
    com.ywt.gapi.system.GetMsgTplListByModuleRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetMsgTplListByModuleRequest::parseFrom,
    com.ywt.gapi.system.GetMsgTplListByModuleResponse::parseFrom);

    private static final StubMethodDescriptor getMsgTplListByModuleProxyAsyncMethod = new StubMethodDescriptor("getMsgTplListByModuleAsync",
    com.ywt.gapi.system.GetMsgTplListByModuleRequest.class, com.ywt.gapi.system.GetMsgTplListByModuleResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetMsgTplListByModuleRequest::parseFrom,
    com.ywt.gapi.system.GetMsgTplListByModuleResponse::parseFrom);

    /**
         * <pre>
         * 得到医生默认或最近添加的地址  create by daiyhua
         * </pre>
         */
    private static final StubMethodDescriptor getDoctorDefaultAddressMethod = new StubMethodDescriptor("getDoctorDefaultAddress",
    com.ywt.gapi.system.GetDoctorAddressRequest.class, com.ywt.gapi.system.GetDoctorDefaultAddressResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDoctorAddressRequest::parseFrom,
    com.ywt.gapi.system.GetDoctorDefaultAddressResponse::parseFrom);

    private static final StubMethodDescriptor getDoctorDefaultAddressAsyncMethod = new StubMethodDescriptor("getDoctorDefaultAddress",
    com.ywt.gapi.system.GetDoctorAddressRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDoctorAddressRequest::parseFrom,
    com.ywt.gapi.system.GetDoctorDefaultAddressResponse::parseFrom);

    private static final StubMethodDescriptor getDoctorDefaultAddressProxyAsyncMethod = new StubMethodDescriptor("getDoctorDefaultAddressAsync",
    com.ywt.gapi.system.GetDoctorAddressRequest.class, com.ywt.gapi.system.GetDoctorDefaultAddressResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDoctorAddressRequest::parseFrom,
    com.ywt.gapi.system.GetDoctorDefaultAddressResponse::parseFrom);

    /**
         * <pre>
         * 同步诊疗卡信息		create by daiyihua
         * </pre>
         */
    private static final StubMethodDescriptor synchronizingInformationMethod = new StubMethodDescriptor("synchronizingInformation",
    com.ywt.gapi.system.SynchronizingInformationRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SynchronizingInformationRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor synchronizingInformationAsyncMethod = new StubMethodDescriptor("synchronizingInformation",
    com.ywt.gapi.system.SynchronizingInformationRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SynchronizingInformationRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor synchronizingInformationProxyAsyncMethod = new StubMethodDescriptor("synchronizingInformationAsync",
    com.ywt.gapi.system.SynchronizingInformationRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SynchronizingInformationRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 根据医院ID得到医院信息		create by daiyihua
         * </pre>
         */
    private static final StubMethodDescriptor getHospitalMethod = new StubMethodDescriptor("getHospital",
    com.ywt.gapi.system.GetHospitalRequest.class, com.ywt.gapi.system.GetHospitalResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalRequest::parseFrom,
    com.ywt.gapi.system.GetHospitalResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalAsyncMethod = new StubMethodDescriptor("getHospital",
    com.ywt.gapi.system.GetHospitalRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalRequest::parseFrom,
    com.ywt.gapi.system.GetHospitalResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalProxyAsyncMethod = new StubMethodDescriptor("getHospitalAsync",
    com.ywt.gapi.system.GetHospitalRequest.class, com.ywt.gapi.system.GetHospitalResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalRequest::parseFrom,
    com.ywt.gapi.system.GetHospitalResponse::parseFrom);

    /**
         * <pre>
         * 同步太和检查结果
         * </pre>
         */
    private static final StubMethodDescriptor syncTaiheInspectionReportMethod = new StubMethodDescriptor("syncTaiheInspectionReport",
    com.ywt.gapi.system.SyncTaiheInspectionReportRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SyncTaiheInspectionReportRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor syncTaiheInspectionReportAsyncMethod = new StubMethodDescriptor("syncTaiheInspectionReport",
    com.ywt.gapi.system.SyncTaiheInspectionReportRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SyncTaiheInspectionReportRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor syncTaiheInspectionReportProxyAsyncMethod = new StubMethodDescriptor("syncTaiheInspectionReportAsync",
    com.ywt.gapi.system.SyncTaiheInspectionReportRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SyncTaiheInspectionReportRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 同步太和检验结果
         * </pre>
         */
    private static final StubMethodDescriptor syncTaiheLaboratoryReportMethod = new StubMethodDescriptor("syncTaiheLaboratoryReport",
    com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor syncTaiheLaboratoryReportAsyncMethod = new StubMethodDescriptor("syncTaiheLaboratoryReport",
    com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor syncTaiheLaboratoryReportProxyAsyncMethod = new StubMethodDescriptor("syncTaiheLaboratoryReportAsync",
    com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * </pre>
         */
    private static final StubMethodDescriptor createHisAddRegisteredFlagMethod = new StubMethodDescriptor("createHisAddRegisteredFlag",
    com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor createHisAddRegisteredFlagAsyncMethod = new StubMethodDescriptor("createHisAddRegisteredFlag",
    com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor createHisAddRegisteredFlagProxyAsyncMethod = new StubMethodDescriptor("createHisAddRegisteredFlagAsync",
    com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 医生团队（展示的）领衔医生所在的一级科室
         * </pre>
         */
    private static final StubMethodDescriptor getTeamDeptListMethod = new StubMethodDescriptor("getTeamDeptList",
    com.ywt.gapi.system.GetTeamDeptListRequest.class, com.ywt.gapi.system.GetTeamDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetTeamDeptListRequest::parseFrom,
    com.ywt.gapi.system.GetTeamDeptListResponse::parseFrom);

    private static final StubMethodDescriptor getTeamDeptListAsyncMethod = new StubMethodDescriptor("getTeamDeptList",
    com.ywt.gapi.system.GetTeamDeptListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetTeamDeptListRequest::parseFrom,
    com.ywt.gapi.system.GetTeamDeptListResponse::parseFrom);

    private static final StubMethodDescriptor getTeamDeptListProxyAsyncMethod = new StubMethodDescriptor("getTeamDeptListAsync",
    com.ywt.gapi.system.GetTeamDeptListRequest.class, com.ywt.gapi.system.GetTeamDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetTeamDeptListRequest::parseFrom,
    com.ywt.gapi.system.GetTeamDeptListResponse::parseFrom);

    /**
         * <pre>
         * 根据条件查找医院列表
         * </pre>
         */
    private static final StubMethodDescriptor findHospitalListMethod = new StubMethodDescriptor("findHospitalList",
    com.ywt.gapi.system.GetHospitalListRequest.class, com.ywt.gapi.system.HospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    private static final StubMethodDescriptor findHospitalListAsyncMethod = new StubMethodDescriptor("findHospitalList",
    com.ywt.gapi.system.GetHospitalListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    private static final StubMethodDescriptor findHospitalListProxyAsyncMethod = new StubMethodDescriptor("findHospitalListAsync",
    com.ywt.gapi.system.GetHospitalListRequest.class, com.ywt.gapi.system.HospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    /**
         * <pre>
         * 得到普通用户关注的医院列表
         * </pre>
         */
    private static final StubMethodDescriptor getGeneralAttentionHospitalListMethod = new StubMethodDescriptor("getGeneralAttentionHospitalList",
    com.ywt.gapi.system.GetHospitalListRequest.class, com.ywt.gapi.system.HospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getGeneralAttentionHospitalListAsyncMethod = new StubMethodDescriptor("getGeneralAttentionHospitalList",
    com.ywt.gapi.system.GetHospitalListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getGeneralAttentionHospitalListProxyAsyncMethod = new StubMethodDescriptor("getGeneralAttentionHospitalListAsync",
    com.ywt.gapi.system.GetHospitalListRequest.class, com.ywt.gapi.system.HospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListRequest::parseFrom,
    com.ywt.gapi.system.HospitalListResponse::parseFrom);

    /**
         * <pre>
         * 保存小程序日志
         * </pre>
         */
    private static final StubMethodDescriptor saveMiniProgramLogMethod = new StubMethodDescriptor("saveMiniProgramLog",
    com.ywt.gapi.system.SaveMiniProgramLogRequest.class, com.ywt.gapi.system.SaveMiniProgramLogResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveMiniProgramLogRequest::parseFrom,
    com.ywt.gapi.system.SaveMiniProgramLogResponse::parseFrom);

    private static final StubMethodDescriptor saveMiniProgramLogAsyncMethod = new StubMethodDescriptor("saveMiniProgramLog",
    com.ywt.gapi.system.SaveMiniProgramLogRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveMiniProgramLogRequest::parseFrom,
    com.ywt.gapi.system.SaveMiniProgramLogResponse::parseFrom);

    private static final StubMethodDescriptor saveMiniProgramLogProxyAsyncMethod = new StubMethodDescriptor("saveMiniProgramLogAsync",
    com.ywt.gapi.system.SaveMiniProgramLogRequest.class, com.ywt.gapi.system.SaveMiniProgramLogResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveMiniProgramLogRequest::parseFrom,
    com.ywt.gapi.system.SaveMiniProgramLogResponse::parseFrom);

    /**
         * <pre>
         * 获取科室
         * </pre>
         */
    private static final StubMethodDescriptor getDeptMethod = new StubMethodDescriptor("getDept",
    com.ywt.gapi.system.GetDeptRequest.class, com.ywt.gapi.system.GetDeptResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDeptRequest::parseFrom,
    com.ywt.gapi.system.GetDeptResponse::parseFrom);

    private static final StubMethodDescriptor getDeptAsyncMethod = new StubMethodDescriptor("getDept",
    com.ywt.gapi.system.GetDeptRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDeptRequest::parseFrom,
    com.ywt.gapi.system.GetDeptResponse::parseFrom);

    private static final StubMethodDescriptor getDeptProxyAsyncMethod = new StubMethodDescriptor("getDeptAsync",
    com.ywt.gapi.system.GetDeptRequest.class, com.ywt.gapi.system.GetDeptResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetDeptRequest::parseFrom,
    com.ywt.gapi.system.GetDeptResponse::parseFrom);

    /**
         * <pre>
         * 得到医务通云平台首页医院列表
         * </pre>
         */
    private static final StubMethodDescriptor getCloudPlatformHospitalListMethod = new StubMethodDescriptor("getCloudPlatformHospitalList",
    com.ywt.gapi.system.HospitalListRequest.class, com.ywt.gapi.system.CloudPlatformHospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalListRequest::parseFrom,
    com.ywt.gapi.system.CloudPlatformHospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getCloudPlatformHospitalListAsyncMethod = new StubMethodDescriptor("getCloudPlatformHospitalList",
    com.ywt.gapi.system.HospitalListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalListRequest::parseFrom,
    com.ywt.gapi.system.CloudPlatformHospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getCloudPlatformHospitalListProxyAsyncMethod = new StubMethodDescriptor("getCloudPlatformHospitalListAsync",
    com.ywt.gapi.system.HospitalListRequest.class, com.ywt.gapi.system.CloudPlatformHospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalListRequest::parseFrom,
    com.ywt.gapi.system.CloudPlatformHospitalListResponse::parseFrom);

    /**
         * <pre>
         *  获取系统字典
         * </pre>
         */
    private static final StubMethodDescriptor getSystemDicTypeItemListByTypeCodeMethod = new StubMethodDescriptor("getSystemDicTypeItemListByTypeCode",
    com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest.class, com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest::parseFrom,
    com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse::parseFrom);

    private static final StubMethodDescriptor getSystemDicTypeItemListByTypeCodeAsyncMethod = new StubMethodDescriptor("getSystemDicTypeItemListByTypeCode",
    com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest::parseFrom,
    com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse::parseFrom);

    private static final StubMethodDescriptor getSystemDicTypeItemListByTypeCodeProxyAsyncMethod = new StubMethodDescriptor("getSystemDicTypeItemListByTypeCodeAsync",
    com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest.class, com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest::parseFrom,
    com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse::parseFrom);

    /**
         * <pre>
         *  清除缓存
         * </pre>
         */
    private static final StubMethodDescriptor invalidateCacheMethod = new StubMethodDescriptor("invalidateCache",
    com.ywt.gapi.system.InvalidateCacheRequest.class, com.ywt.gapi.system.InvalidateCacheResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.InvalidateCacheRequest::parseFrom,
    com.ywt.gapi.system.InvalidateCacheResponse::parseFrom);

    private static final StubMethodDescriptor invalidateCacheAsyncMethod = new StubMethodDescriptor("invalidateCache",
    com.ywt.gapi.system.InvalidateCacheRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.InvalidateCacheRequest::parseFrom,
    com.ywt.gapi.system.InvalidateCacheResponse::parseFrom);

    private static final StubMethodDescriptor invalidateCacheProxyAsyncMethod = new StubMethodDescriptor("invalidateCacheAsync",
    com.ywt.gapi.system.InvalidateCacheRequest.class, com.ywt.gapi.system.InvalidateCacheResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.InvalidateCacheRequest::parseFrom,
    com.ywt.gapi.system.InvalidateCacheResponse::parseFrom);

    /**
         * <pre>
         *  获取缓存 keys
         * </pre>
         */
    private static final StubMethodDescriptor getCacheKeysMethod = new StubMethodDescriptor("getCacheKeys",
    com.ywt.gapi.system.GetCacheKeysRequest.class, com.ywt.gapi.system.GetCacheKeysResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetCacheKeysRequest::parseFrom,
    com.ywt.gapi.system.GetCacheKeysResponse::parseFrom);

    private static final StubMethodDescriptor getCacheKeysAsyncMethod = new StubMethodDescriptor("getCacheKeys",
    com.ywt.gapi.system.GetCacheKeysRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetCacheKeysRequest::parseFrom,
    com.ywt.gapi.system.GetCacheKeysResponse::parseFrom);

    private static final StubMethodDescriptor getCacheKeysProxyAsyncMethod = new StubMethodDescriptor("getCacheKeysAsync",
    com.ywt.gapi.system.GetCacheKeysRequest.class, com.ywt.gapi.system.GetCacheKeysResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetCacheKeysRequest::parseFrom,
    com.ywt.gapi.system.GetCacheKeysResponse::parseFrom);

    /**
         * <pre>
         *  获取配置的内容
         * </pre>
         */
    private static final StubMethodDescriptor getConfigByTypeMethod = new StubMethodDescriptor("getConfigByType",
    com.ywt.gapi.system.GetConfigByTypeRequest.class, com.ywt.gapi.system.GetConfigByTypeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetConfigByTypeRequest::parseFrom,
    com.ywt.gapi.system.GetConfigByTypeResponse::parseFrom);

    private static final StubMethodDescriptor getConfigByTypeAsyncMethod = new StubMethodDescriptor("getConfigByType",
    com.ywt.gapi.system.GetConfigByTypeRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetConfigByTypeRequest::parseFrom,
    com.ywt.gapi.system.GetConfigByTypeResponse::parseFrom);

    private static final StubMethodDescriptor getConfigByTypeProxyAsyncMethod = new StubMethodDescriptor("getConfigByTypeAsync",
    com.ywt.gapi.system.GetConfigByTypeRequest.class, com.ywt.gapi.system.GetConfigByTypeResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetConfigByTypeRequest::parseFrom,
    com.ywt.gapi.system.GetConfigByTypeResponse::parseFrom);

    /**
         * <pre>
         *  根据医院IDs得到医院信息		
         * </pre>
         */
    private static final StubMethodDescriptor getHospitalByHospitalIdsMethod = new StubMethodDescriptor("getHospitalByHospitalIds",
    com.ywt.gapi.system.GetHospitalByHospitalIdsRequest.class, com.ywt.gapi.system.GetHospitalByHospitalIdsResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalByHospitalIdsRequest::parseFrom,
    com.ywt.gapi.system.GetHospitalByHospitalIdsResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalByHospitalIdsAsyncMethod = new StubMethodDescriptor("getHospitalByHospitalIds",
    com.ywt.gapi.system.GetHospitalByHospitalIdsRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalByHospitalIdsRequest::parseFrom,
    com.ywt.gapi.system.GetHospitalByHospitalIdsResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalByHospitalIdsProxyAsyncMethod = new StubMethodDescriptor("getHospitalByHospitalIdsAsync",
    com.ywt.gapi.system.GetHospitalByHospitalIdsRequest.class, com.ywt.gapi.system.GetHospitalByHospitalIdsResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalByHospitalIdsRequest::parseFrom,
    com.ywt.gapi.system.GetHospitalByHospitalIdsResponse::parseFrom);

    /**
         * <pre>
         *  保存GPS记录
         * </pre>
         */
    private static final StubMethodDescriptor saveGPSRecordMethod = new StubMethodDescriptor("saveGPSRecord",
    com.ywt.gapi.system.SaveGPSRecordRequest.class, com.ywt.gapi.system.SaveGPSRecordResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveGPSRecordRequest::parseFrom,
    com.ywt.gapi.system.SaveGPSRecordResponse::parseFrom);

    private static final StubMethodDescriptor saveGPSRecordAsyncMethod = new StubMethodDescriptor("saveGPSRecord",
    com.ywt.gapi.system.SaveGPSRecordRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveGPSRecordRequest::parseFrom,
    com.ywt.gapi.system.SaveGPSRecordResponse::parseFrom);

    private static final StubMethodDescriptor saveGPSRecordProxyAsyncMethod = new StubMethodDescriptor("saveGPSRecordAsync",
    com.ywt.gapi.system.SaveGPSRecordRequest.class, com.ywt.gapi.system.SaveGPSRecordResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveGPSRecordRequest::parseFrom,
    com.ywt.gapi.system.SaveGPSRecordResponse::parseFrom);

    /**
         * <pre>
         *  根据配置ID获取配置信息
         * </pre>
         */
    private static final StubMethodDescriptor getConfigByIdMethod = new StubMethodDescriptor("getConfigById",
    com.ywt.gapi.system.GetConfigByIdRequest.class, com.ywt.gapi.system.GetConfigByIdResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetConfigByIdRequest::parseFrom,
    com.ywt.gapi.system.GetConfigByIdResponse::parseFrom);

    private static final StubMethodDescriptor getConfigByIdAsyncMethod = new StubMethodDescriptor("getConfigById",
    com.ywt.gapi.system.GetConfigByIdRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetConfigByIdRequest::parseFrom,
    com.ywt.gapi.system.GetConfigByIdResponse::parseFrom);

    private static final StubMethodDescriptor getConfigByIdProxyAsyncMethod = new StubMethodDescriptor("getConfigByIdAsync",
    com.ywt.gapi.system.GetConfigByIdRequest.class, com.ywt.gapi.system.GetConfigByIdResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetConfigByIdRequest::parseFrom,
    com.ywt.gapi.system.GetConfigByIdResponse::parseFrom);

    /**
         * <pre>
         *  根据名称获取科室列表
         * </pre>
         */
    private static final StubMethodDescriptor searchDeptListByNameMethod = new StubMethodDescriptor("searchDeptListByName",
    com.ywt.gapi.system.SearchDeptListByNameRequest.class, com.ywt.gapi.system.SearchDeptListByNameResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDeptListByNameRequest::parseFrom,
    com.ywt.gapi.system.SearchDeptListByNameResponse::parseFrom);

    private static final StubMethodDescriptor searchDeptListByNameAsyncMethod = new StubMethodDescriptor("searchDeptListByName",
    com.ywt.gapi.system.SearchDeptListByNameRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDeptListByNameRequest::parseFrom,
    com.ywt.gapi.system.SearchDeptListByNameResponse::parseFrom);

    private static final StubMethodDescriptor searchDeptListByNameProxyAsyncMethod = new StubMethodDescriptor("searchDeptListByNameAsync",
    com.ywt.gapi.system.SearchDeptListByNameRequest.class, com.ywt.gapi.system.SearchDeptListByNameResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDeptListByNameRequest::parseFrom,
    com.ywt.gapi.system.SearchDeptListByNameResponse::parseFrom);

    /**
         * <pre>
         *  根据名称获取医生列表
         * </pre>
         */
    private static final StubMethodDescriptor searchDoctorListByNameMethod = new StubMethodDescriptor("searchDoctorListByName",
    com.ywt.gapi.system.SearchDoctorListByNameRequest.class, com.ywt.gapi.system.SearchDoctorListByNameResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDoctorListByNameRequest::parseFrom,
    com.ywt.gapi.system.SearchDoctorListByNameResponse::parseFrom);

    private static final StubMethodDescriptor searchDoctorListByNameAsyncMethod = new StubMethodDescriptor("searchDoctorListByName",
    com.ywt.gapi.system.SearchDoctorListByNameRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDoctorListByNameRequest::parseFrom,
    com.ywt.gapi.system.SearchDoctorListByNameResponse::parseFrom);

    private static final StubMethodDescriptor searchDoctorListByNameProxyAsyncMethod = new StubMethodDescriptor("searchDoctorListByNameAsync",
    com.ywt.gapi.system.SearchDoctorListByNameRequest.class, com.ywt.gapi.system.SearchDoctorListByNameResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDoctorListByNameRequest::parseFrom,
    com.ywt.gapi.system.SearchDoctorListByNameResponse::parseFrom);

    /**
         * <pre>
         *  根据姓名从多个医院查找医生
         * </pre>
         */
    private static final StubMethodDescriptor searchDoctorByNameFromMultiHospMethod = new StubMethodDescriptor("searchDoctorByNameFromMultiHosp",
    com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest.class, com.ywt.gapi.system.SearchDoctorListByNameResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest::parseFrom,
    com.ywt.gapi.system.SearchDoctorListByNameResponse::parseFrom);

    private static final StubMethodDescriptor searchDoctorByNameFromMultiHospAsyncMethod = new StubMethodDescriptor("searchDoctorByNameFromMultiHosp",
    com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest::parseFrom,
    com.ywt.gapi.system.SearchDoctorListByNameResponse::parseFrom);

    private static final StubMethodDescriptor searchDoctorByNameFromMultiHospProxyAsyncMethod = new StubMethodDescriptor("searchDoctorByNameFromMultiHospAsync",
    com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest.class, com.ywt.gapi.system.SearchDoctorListByNameResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest::parseFrom,
    com.ywt.gapi.system.SearchDoctorListByNameResponse::parseFrom);

    /**
         * <pre>
         *  得到预约挂号公告信息
         * </pre>
         */
    private static final StubMethodDescriptor getRegisteredAnnouncementMethod = new StubMethodDescriptor("getRegisteredAnnouncement",
    com.ywt.gapi.system.RetRegisteredAnnouncementRequest.class, com.ywt.gapi.system.RetRegisteredAnnouncementResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.RetRegisteredAnnouncementRequest::parseFrom,
    com.ywt.gapi.system.RetRegisteredAnnouncementResponse::parseFrom);

    private static final StubMethodDescriptor getRegisteredAnnouncementAsyncMethod = new StubMethodDescriptor("getRegisteredAnnouncement",
    com.ywt.gapi.system.RetRegisteredAnnouncementRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.RetRegisteredAnnouncementRequest::parseFrom,
    com.ywt.gapi.system.RetRegisteredAnnouncementResponse::parseFrom);

    private static final StubMethodDescriptor getRegisteredAnnouncementProxyAsyncMethod = new StubMethodDescriptor("getRegisteredAnnouncementAsync",
    com.ywt.gapi.system.RetRegisteredAnnouncementRequest.class, com.ywt.gapi.system.RetRegisteredAnnouncementResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.RetRegisteredAnnouncementRequest::parseFrom,
    com.ywt.gapi.system.RetRegisteredAnnouncementResponse::parseFrom);

    /**
         * <pre>
         *  判断是否有救护车的权限
         * </pre>
         */
    private static final StubMethodDescriptor checkHasAmbulanceRightMethod = new StubMethodDescriptor("checkHasAmbulanceRight",
    com.ywt.gapi.system.CheckHasAmbulanceRightRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CheckHasAmbulanceRightRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor checkHasAmbulanceRightAsyncMethod = new StubMethodDescriptor("checkHasAmbulanceRight",
    com.ywt.gapi.system.CheckHasAmbulanceRightRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CheckHasAmbulanceRightRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor checkHasAmbulanceRightProxyAsyncMethod = new StubMethodDescriptor("checkHasAmbulanceRightAsync",
    com.ywt.gapi.system.CheckHasAmbulanceRightRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CheckHasAmbulanceRightRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         *  记录救护车位置
         * </pre>
         */
    private static final StubMethodDescriptor saveAmbulanceLocationMethod = new StubMethodDescriptor("saveAmbulanceLocation",
    com.ywt.gapi.system.SaveAmbulanceLocationRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAmbulanceLocationRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor saveAmbulanceLocationAsyncMethod = new StubMethodDescriptor("saveAmbulanceLocation",
    com.ywt.gapi.system.SaveAmbulanceLocationRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAmbulanceLocationRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    private static final StubMethodDescriptor saveAmbulanceLocationProxyAsyncMethod = new StubMethodDescriptor("saveAmbulanceLocationAsync",
    com.ywt.gapi.system.SaveAmbulanceLocationRequest.class, com.ywt.gapi.Result.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.SaveAmbulanceLocationRequest::parseFrom,
    com.ywt.gapi.Result::parseFrom);

    /**
         * <pre>
         * 须知展示
         * </pre>
         */
    private static final StubMethodDescriptor showCommonAnnouncementMethod = new StubMethodDescriptor("showCommonAnnouncement",
    com.ywt.gapi.system.CommonAnnouncementRequest.class, com.ywt.gapi.system.CommonAnnouncementResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CommonAnnouncementRequest::parseFrom,
    com.ywt.gapi.system.CommonAnnouncementResponse::parseFrom);

    private static final StubMethodDescriptor showCommonAnnouncementAsyncMethod = new StubMethodDescriptor("showCommonAnnouncement",
    com.ywt.gapi.system.CommonAnnouncementRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CommonAnnouncementRequest::parseFrom,
    com.ywt.gapi.system.CommonAnnouncementResponse::parseFrom);

    private static final StubMethodDescriptor showCommonAnnouncementProxyAsyncMethod = new StubMethodDescriptor("showCommonAnnouncementAsync",
    com.ywt.gapi.system.CommonAnnouncementRequest.class, com.ywt.gapi.system.CommonAnnouncementResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.CommonAnnouncementRequest::parseFrom,
    com.ywt.gapi.system.CommonAnnouncementResponse::parseFrom);

    /**
         * <pre>
         * 就诊指南
         * </pre>
         */
    private static final StubMethodDescriptor getGuideListMethod = new StubMethodDescriptor("getGuideList",
    com.ywt.gapi.system.GuideListRequest.class, com.ywt.gapi.system.GuideListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GuideListRequest::parseFrom,
    com.ywt.gapi.system.GuideListResponse::parseFrom);

    private static final StubMethodDescriptor getGuideListAsyncMethod = new StubMethodDescriptor("getGuideList",
    com.ywt.gapi.system.GuideListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GuideListRequest::parseFrom,
    com.ywt.gapi.system.GuideListResponse::parseFrom);

    private static final StubMethodDescriptor getGuideListProxyAsyncMethod = new StubMethodDescriptor("getGuideListAsync",
    com.ywt.gapi.system.GuideListRequest.class, com.ywt.gapi.system.GuideListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GuideListRequest::parseFrom,
    com.ywt.gapi.system.GuideListResponse::parseFrom);

    /**
         * <pre>
         *  根据条件获取医院列表
         * </pre>
         */
    private static final StubMethodDescriptor getHospitalListByTypeMethod = new StubMethodDescriptor("getHospitalListByType",
    com.ywt.gapi.system.GetHospitalListByTypeRequest.class, com.ywt.gapi.system.AreaHospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListByTypeRequest::parseFrom,
    com.ywt.gapi.system.AreaHospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalListByTypeAsyncMethod = new StubMethodDescriptor("getHospitalListByType",
    com.ywt.gapi.system.GetHospitalListByTypeRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListByTypeRequest::parseFrom,
    com.ywt.gapi.system.AreaHospitalListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalListByTypeProxyAsyncMethod = new StubMethodDescriptor("getHospitalListByTypeAsync",
    com.ywt.gapi.system.GetHospitalListByTypeRequest.class, com.ywt.gapi.system.AreaHospitalListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.GetHospitalListByTypeRequest::parseFrom,
    com.ywt.gapi.system.AreaHospitalListResponse::parseFrom);

    /**
         * <pre>
         * 科室介绍
         * </pre>
         */
    private static final StubMethodDescriptor getIntroduceMethod = new StubMethodDescriptor("getIntroduce",
    com.ywt.gapi.system.IntroduceRequest.class, com.ywt.gapi.system.IntroduceResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.IntroduceRequest::parseFrom,
    com.ywt.gapi.system.IntroduceResponse::parseFrom);

    private static final StubMethodDescriptor getIntroduceAsyncMethod = new StubMethodDescriptor("getIntroduce",
    com.ywt.gapi.system.IntroduceRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.IntroduceRequest::parseFrom,
    com.ywt.gapi.system.IntroduceResponse::parseFrom);

    private static final StubMethodDescriptor getIntroduceProxyAsyncMethod = new StubMethodDescriptor("getIntroduceAsync",
    com.ywt.gapi.system.IntroduceRequest.class, com.ywt.gapi.system.IntroduceResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.IntroduceRequest::parseFrom,
    com.ywt.gapi.system.IntroduceResponse::parseFrom);

    /**
         * <pre>
         * 住院科室列表
         * </pre>
         */
    private static final StubMethodDescriptor hospitalMedicalDeptListMethod = new StubMethodDescriptor("hospitalMedicalDeptList",
    com.ywt.gapi.system.HospitalMedicalDeptListRequest.class, com.ywt.gapi.system.HospitalMedicalDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalMedicalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalMedicalDeptListResponse::parseFrom);

    private static final StubMethodDescriptor hospitalMedicalDeptListAsyncMethod = new StubMethodDescriptor("hospitalMedicalDeptList",
    com.ywt.gapi.system.HospitalMedicalDeptListRequest.class, java.util.concurrent.CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalMedicalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalMedicalDeptListResponse::parseFrom);

    private static final StubMethodDescriptor hospitalMedicalDeptListProxyAsyncMethod = new StubMethodDescriptor("hospitalMedicalDeptListAsync",
    com.ywt.gapi.system.HospitalMedicalDeptListRequest.class, com.ywt.gapi.system.HospitalMedicalDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), com.ywt.gapi.system.HospitalMedicalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalMedicalDeptListResponse::parseFrom);





    public static class SystemServiceStub implements SystemService{
        private final Invoker<SystemService> invoker;

        public SystemServiceStub(Invoker<SystemService> invoker) {
            this.invoker = invoker;
        }

            /**
         * <pre>
         * 医院Id
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.HospitalListResponse getHospitalList(com.ywt.gapi.system.HospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.HospitalListResponse> getHospitalListAsync(com.ywt.gapi.system.HospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalListAsyncMethod, request);
        }

            /**
         * <pre>
         * 医院Id
         * </pre>
         */
        @Override
        public void getHospitalList(com.ywt.gapi.system.HospitalListRequest request, StreamObserver<com.ywt.gapi.system.HospitalListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getHospitalListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 医院名称
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.DeptListResponse getDeptList(com.ywt.gapi.system.DeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDeptListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.DeptListResponse> getDeptListAsync(com.ywt.gapi.system.DeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDeptListAsyncMethod, request);
        }

            /**
         * <pre>
         * 医院名称
         * </pre>
         */
        @Override
        public void getDeptList(com.ywt.gapi.system.DeptListRequest request, StreamObserver<com.ywt.gapi.system.DeptListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getDeptListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 地区Id
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.HospitalDeptListResponse getHospitalDeptList(com.ywt.gapi.system.HospitalDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalDeptListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.HospitalDeptListResponse> getHospitalDeptListAsync(com.ywt.gapi.system.HospitalDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalDeptListAsyncMethod, request);
        }

            /**
         * <pre>
         * 地区Id
         * </pre>
         */
        @Override
        public void getHospitalDeptList(com.ywt.gapi.system.HospitalDeptListRequest request, StreamObserver<com.ywt.gapi.system.HospitalDeptListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getHospitalDeptListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 医院等级
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.CheckAppVersionResponse checkAppVersion(com.ywt.gapi.system.CheckAppVersionRequest request){
            return StubInvocationUtil.unaryCall(invoker, checkAppVersionMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.CheckAppVersionResponse> checkAppVersionAsync(com.ywt.gapi.system.CheckAppVersionRequest request){
            return StubInvocationUtil.unaryCall(invoker, checkAppVersionAsyncMethod, request);
        }

            /**
         * <pre>
         * 医院等级
         * </pre>
         */
        @Override
        public void checkAppVersion(com.ywt.gapi.system.CheckAppVersionRequest request, StreamObserver<com.ywt.gapi.system.CheckAppVersionResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, checkAppVersionMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 医院Logo
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetAreaListResponse getAreaList(com.ywt.gapi.system.GetAreaListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAreaListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetAreaListResponse> getAreaListAsync(com.ywt.gapi.system.GetAreaListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAreaListAsyncMethod, request);
        }

            /**
         * <pre>
         * 医院Logo
         * </pre>
         */
        @Override
        public void getAreaList(com.ywt.gapi.system.GetAreaListRequest request, StreamObserver<com.ywt.gapi.system.GetAreaListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAreaListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 详细地址
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetAddressListResponse getAddressList(com.ywt.gapi.system.GetAddressListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAddressListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetAddressListResponse> getAddressListAsync(com.ywt.gapi.system.GetAddressListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAddressListAsyncMethod, request);
        }

            /**
         * <pre>
         * 详细地址
         * </pre>
         */
        @Override
        public void getAddressList(com.ywt.gapi.system.GetAddressListRequest request, StreamObserver<com.ywt.gapi.system.GetAddressListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAddressListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 能否挂号
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.SaveAddressResponse saveAddress(com.ywt.gapi.system.SaveAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveAddressMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.SaveAddressResponse> saveAddressAsync(com.ywt.gapi.system.SaveAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveAddressAsyncMethod, request);
        }

            /**
         * <pre>
         * 能否挂号
         * </pre>
         */
        @Override
        public void saveAddress(com.ywt.gapi.system.SaveAddressRequest request, StreamObserver<com.ywt.gapi.system.SaveAddressResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveAddressMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 状态(暂用于表示用户是否关注)
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result deleteAddress(com.ywt.gapi.system.DeleteAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, deleteAddressMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> deleteAddressAsync(com.ywt.gapi.system.DeleteAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, deleteAddressAsyncMethod, request);
        }

            /**
         * <pre>
         * 状态(暂用于表示用户是否关注)
         * </pre>
         */
        @Override
        public void deleteAddress(com.ywt.gapi.system.DeleteAddressRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, deleteAddressMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 父ID
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetAddressResponse getAddress(com.ywt.gapi.system.GetAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAddressMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetAddressResponse> getAddressAsync(com.ywt.gapi.system.GetAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAddressAsyncMethod, request);
        }

            /**
         * <pre>
         * 父ID
         * </pre>
         */
        @Override
        public void getAddress(com.ywt.gapi.system.GetAddressRequest request, StreamObserver<com.ywt.gapi.system.GetAddressResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAddressMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 状态
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetAreaTreeNodeListResponse getAreaTreeNodeList(com.ywt.gapi.system.GetAreaTreeNodeListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAreaTreeNodeListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetAreaTreeNodeListResponse> getAreaTreeNodeListAsync(com.ywt.gapi.system.GetAreaTreeNodeListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAreaTreeNodeListAsyncMethod, request);
        }

            /**
         * <pre>
         * 状态
         * </pre>
         */
        @Override
        public void getAreaTreeNodeList(com.ywt.gapi.system.GetAreaTreeNodeListRequest request, StreamObserver<com.ywt.gapi.system.GetAreaTreeNodeListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAreaTreeNodeListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 添加时间
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.AreaListResponse getAppAreaList(com.ywt.gapi.system.AreaListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAppAreaListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.AreaListResponse> getAppAreaListAsync(com.ywt.gapi.system.AreaListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAppAreaListAsyncMethod, request);
        }

            /**
         * <pre>
         * 添加时间
         * </pre>
         */
        @Override
        public void getAppAreaList(com.ywt.gapi.system.AreaListRequest request, StreamObserver<com.ywt.gapi.system.AreaListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAppAreaListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 修改时间
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.AreaHospitalListResponse getAreaHospitalList(com.ywt.gapi.system.AreaHospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAreaHospitalListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.AreaHospitalListResponse> getAreaHospitalListAsync(com.ywt.gapi.system.AreaHospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAreaHospitalListAsyncMethod, request);
        }

            /**
         * <pre>
         * 修改时间
         * </pre>
         */
        @Override
        public void getAreaHospitalList(com.ywt.gapi.system.AreaHospitalListRequest request, StreamObserver<com.ywt.gapi.system.AreaHospitalListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAreaHospitalListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 省Id
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetWeChatConfigResponse getWeChatConfig(com.ywt.gapi.system.GetWeChatConfigRequest request){
            return StubInvocationUtil.unaryCall(invoker, getWeChatConfigMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetWeChatConfigResponse> getWeChatConfigAsync(com.ywt.gapi.system.GetWeChatConfigRequest request){
            return StubInvocationUtil.unaryCall(invoker, getWeChatConfigAsyncMethod, request);
        }

            /**
         * <pre>
         * 省Id
         * </pre>
         */
        @Override
        public void getWeChatConfig(com.ywt.gapi.system.GetWeChatConfigRequest request, StreamObserver<com.ywt.gapi.system.GetWeChatConfigResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getWeChatConfigMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 市Id
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetDeptByYiDaCodeResponse getDeptByYiDaCode(com.ywt.gapi.system.GetDeptByYiDaCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDeptByYiDaCodeMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetDeptByYiDaCodeResponse> getDeptByYiDaCodeAsync(com.ywt.gapi.system.GetDeptByYiDaCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDeptByYiDaCodeAsyncMethod, request);
        }

            /**
         * <pre>
         * 市Id
         * </pre>
         */
        @Override
        public void getDeptByYiDaCode(com.ywt.gapi.system.GetDeptByYiDaCodeRequest request, StreamObserver<com.ywt.gapi.system.GetDeptByYiDaCodeResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getDeptByYiDaCodeMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 区Id
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetYiDaDoctorCodeResponse getYiDaDoctorCode(com.ywt.gapi.system.GetYiDaDoctorCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getYiDaDoctorCodeMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetYiDaDoctorCodeResponse> getYiDaDoctorCodeAsync(com.ywt.gapi.system.GetYiDaDoctorCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getYiDaDoctorCodeAsyncMethod, request);
        }

            /**
         * <pre>
         * 区Id
         * </pre>
         */
        @Override
        public void getYiDaDoctorCode(com.ywt.gapi.system.GetYiDaDoctorCodeRequest request, StreamObserver<com.ywt.gapi.system.GetYiDaDoctorCodeResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getYiDaDoctorCodeMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 获取国际疾病分类
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetIcdCodeListResponse getIcdCodeList(com.ywt.gapi.system.GetIcdCodeListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getIcdCodeListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetIcdCodeListResponse> getIcdCodeListAsync(com.ywt.gapi.system.GetIcdCodeListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getIcdCodeListAsyncMethod, request);
        }

            /**
         * <pre>
         * 获取国际疾病分类
         * </pre>
         */
        @Override
        public void getIcdCodeList(com.ywt.gapi.system.GetIcdCodeListRequest request, StreamObserver<com.ywt.gapi.system.GetIcdCodeListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getIcdCodeListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 得到所有地区集合，包括区、县
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetAreaListResponse getAllAreaList(com.ywt.gapi.system.GetAreaListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAllAreaListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetAreaListResponse> getAllAreaListAsync(com.ywt.gapi.system.GetAreaListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getAllAreaListAsyncMethod, request);
        }

            /**
         * <pre>
         * 得到所有地区集合，包括区、县
         * </pre>
         */
        @Override
        public void getAllAreaList(com.ywt.gapi.system.GetAreaListRequest request, StreamObserver<com.ywt.gapi.system.GetAreaListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getAllAreaListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 获取医生地址列表 create by daiyihua
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetAddressListResponse getDoctorAddressList(com.ywt.gapi.system.GetAddressListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDoctorAddressListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetAddressListResponse> getDoctorAddressListAsync(com.ywt.gapi.system.GetAddressListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDoctorAddressListAsyncMethod, request);
        }

            /**
         * <pre>
         * 获取医生地址列表 create by daiyihua
         * </pre>
         */
        @Override
        public void getDoctorAddressList(com.ywt.gapi.system.GetAddressListRequest request, StreamObserver<com.ywt.gapi.system.GetAddressListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getDoctorAddressListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 医生保存地址 create by daiyihua
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result doctorSaveAddress(com.ywt.gapi.system.SaveAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, doctorSaveAddressMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> doctorSaveAddressAsync(com.ywt.gapi.system.SaveAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, doctorSaveAddressAsyncMethod, request);
        }

            /**
         * <pre>
         * 医生保存地址 create by daiyihua
         * </pre>
         */
        @Override
        public void doctorSaveAddress(com.ywt.gapi.system.SaveAddressRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, doctorSaveAddressMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 医生设置默认地址 create by daiyihua
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result settingDefaultAddress(com.ywt.gapi.system.SettingDefaultAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, settingDefaultAddressMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> settingDefaultAddressAsync(com.ywt.gapi.system.SettingDefaultAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, settingDefaultAddressAsyncMethod, request);
        }

            /**
         * <pre>
         * 医生设置默认地址 create by daiyihua
         * </pre>
         */
        @Override
        public void settingDefaultAddress(com.ywt.gapi.system.SettingDefaultAddressRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, settingDefaultAddressMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 获取消息模板列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetMsgTplListByModuleResponse getMsgTplListByModule(com.ywt.gapi.system.GetMsgTplListByModuleRequest request){
            return StubInvocationUtil.unaryCall(invoker, getMsgTplListByModuleMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetMsgTplListByModuleResponse> getMsgTplListByModuleAsync(com.ywt.gapi.system.GetMsgTplListByModuleRequest request){
            return StubInvocationUtil.unaryCall(invoker, getMsgTplListByModuleAsyncMethod, request);
        }

            /**
         * <pre>
         * 获取消息模板列表
         * </pre>
         */
        @Override
        public void getMsgTplListByModule(com.ywt.gapi.system.GetMsgTplListByModuleRequest request, StreamObserver<com.ywt.gapi.system.GetMsgTplListByModuleResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getMsgTplListByModuleMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 得到医生默认或最近添加的地址  create by daiyhua
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetDoctorDefaultAddressResponse getDoctorDefaultAddress(com.ywt.gapi.system.GetDoctorAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDoctorDefaultAddressMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetDoctorDefaultAddressResponse> getDoctorDefaultAddressAsync(com.ywt.gapi.system.GetDoctorAddressRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDoctorDefaultAddressAsyncMethod, request);
        }

            /**
         * <pre>
         * 得到医生默认或最近添加的地址  create by daiyhua
         * </pre>
         */
        @Override
        public void getDoctorDefaultAddress(com.ywt.gapi.system.GetDoctorAddressRequest request, StreamObserver<com.ywt.gapi.system.GetDoctorDefaultAddressResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getDoctorDefaultAddressMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 同步诊疗卡信息		create by daiyihua
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result synchronizingInformation(com.ywt.gapi.system.SynchronizingInformationRequest request){
            return StubInvocationUtil.unaryCall(invoker, synchronizingInformationMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> synchronizingInformationAsync(com.ywt.gapi.system.SynchronizingInformationRequest request){
            return StubInvocationUtil.unaryCall(invoker, synchronizingInformationAsyncMethod, request);
        }

            /**
         * <pre>
         * 同步诊疗卡信息		create by daiyihua
         * </pre>
         */
        @Override
        public void synchronizingInformation(com.ywt.gapi.system.SynchronizingInformationRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, synchronizingInformationMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 根据医院ID得到医院信息		create by daiyihua
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetHospitalResponse getHospital(com.ywt.gapi.system.GetHospitalRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetHospitalResponse> getHospitalAsync(com.ywt.gapi.system.GetHospitalRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalAsyncMethod, request);
        }

            /**
         * <pre>
         * 根据医院ID得到医院信息		create by daiyihua
         * </pre>
         */
        @Override
        public void getHospital(com.ywt.gapi.system.GetHospitalRequest request, StreamObserver<com.ywt.gapi.system.GetHospitalResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getHospitalMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 同步太和检查结果
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result syncTaiheInspectionReport(com.ywt.gapi.system.SyncTaiheInspectionReportRequest request){
            return StubInvocationUtil.unaryCall(invoker, syncTaiheInspectionReportMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> syncTaiheInspectionReportAsync(com.ywt.gapi.system.SyncTaiheInspectionReportRequest request){
            return StubInvocationUtil.unaryCall(invoker, syncTaiheInspectionReportAsyncMethod, request);
        }

            /**
         * <pre>
         * 同步太和检查结果
         * </pre>
         */
        @Override
        public void syncTaiheInspectionReport(com.ywt.gapi.system.SyncTaiheInspectionReportRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, syncTaiheInspectionReportMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 同步太和检验结果
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result syncTaiheLaboratoryReport(com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest request){
            return StubInvocationUtil.unaryCall(invoker, syncTaiheLaboratoryReportMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> syncTaiheLaboratoryReportAsync(com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest request){
            return StubInvocationUtil.unaryCall(invoker, syncTaiheLaboratoryReportAsyncMethod, request);
        }

            /**
         * <pre>
         * 同步太和检验结果
         * </pre>
         */
        @Override
        public void syncTaiheLaboratoryReport(com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, syncTaiheLaboratoryReportMethod , request, responseObserver);
        }
            /**
         * <pre>
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result createHisAddRegisteredFlag(com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest request){
            return StubInvocationUtil.unaryCall(invoker, createHisAddRegisteredFlagMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> createHisAddRegisteredFlagAsync(com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest request){
            return StubInvocationUtil.unaryCall(invoker, createHisAddRegisteredFlagAsyncMethod, request);
        }

            /**
         * <pre>
         * </pre>
         */
        @Override
        public void createHisAddRegisteredFlag(com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, createHisAddRegisteredFlagMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 医生团队（展示的）领衔医生所在的一级科室
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetTeamDeptListResponse getTeamDeptList(com.ywt.gapi.system.GetTeamDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getTeamDeptListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetTeamDeptListResponse> getTeamDeptListAsync(com.ywt.gapi.system.GetTeamDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getTeamDeptListAsyncMethod, request);
        }

            /**
         * <pre>
         * 医生团队（展示的）领衔医生所在的一级科室
         * </pre>
         */
        @Override
        public void getTeamDeptList(com.ywt.gapi.system.GetTeamDeptListRequest request, StreamObserver<com.ywt.gapi.system.GetTeamDeptListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getTeamDeptListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 根据条件查找医院列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.HospitalListResponse findHospitalList(com.ywt.gapi.system.GetHospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, findHospitalListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.HospitalListResponse> findHospitalListAsync(com.ywt.gapi.system.GetHospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, findHospitalListAsyncMethod, request);
        }

            /**
         * <pre>
         * 根据条件查找医院列表
         * </pre>
         */
        @Override
        public void findHospitalList(com.ywt.gapi.system.GetHospitalListRequest request, StreamObserver<com.ywt.gapi.system.HospitalListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, findHospitalListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 得到普通用户关注的医院列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.HospitalListResponse getGeneralAttentionHospitalList(com.ywt.gapi.system.GetHospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getGeneralAttentionHospitalListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.HospitalListResponse> getGeneralAttentionHospitalListAsync(com.ywt.gapi.system.GetHospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getGeneralAttentionHospitalListAsyncMethod, request);
        }

            /**
         * <pre>
         * 得到普通用户关注的医院列表
         * </pre>
         */
        @Override
        public void getGeneralAttentionHospitalList(com.ywt.gapi.system.GetHospitalListRequest request, StreamObserver<com.ywt.gapi.system.HospitalListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getGeneralAttentionHospitalListMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 保存小程序日志
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.SaveMiniProgramLogResponse saveMiniProgramLog(com.ywt.gapi.system.SaveMiniProgramLogRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveMiniProgramLogMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.SaveMiniProgramLogResponse> saveMiniProgramLogAsync(com.ywt.gapi.system.SaveMiniProgramLogRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveMiniProgramLogAsyncMethod, request);
        }

            /**
         * <pre>
         * 保存小程序日志
         * </pre>
         */
        @Override
        public void saveMiniProgramLog(com.ywt.gapi.system.SaveMiniProgramLogRequest request, StreamObserver<com.ywt.gapi.system.SaveMiniProgramLogResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveMiniProgramLogMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 获取科室
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetDeptResponse getDept(com.ywt.gapi.system.GetDeptRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDeptMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetDeptResponse> getDeptAsync(com.ywt.gapi.system.GetDeptRequest request){
            return StubInvocationUtil.unaryCall(invoker, getDeptAsyncMethod, request);
        }

            /**
         * <pre>
         * 获取科室
         * </pre>
         */
        @Override
        public void getDept(com.ywt.gapi.system.GetDeptRequest request, StreamObserver<com.ywt.gapi.system.GetDeptResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getDeptMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 得到医务通云平台首页医院列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.CloudPlatformHospitalListResponse getCloudPlatformHospitalList(com.ywt.gapi.system.HospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getCloudPlatformHospitalListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.CloudPlatformHospitalListResponse> getCloudPlatformHospitalListAsync(com.ywt.gapi.system.HospitalListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getCloudPlatformHospitalListAsyncMethod, request);
        }

            /**
         * <pre>
         * 得到医务通云平台首页医院列表
         * </pre>
         */
        @Override
        public void getCloudPlatformHospitalList(com.ywt.gapi.system.HospitalListRequest request, StreamObserver<com.ywt.gapi.system.CloudPlatformHospitalListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getCloudPlatformHospitalListMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  获取系统字典
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse getSystemDicTypeItemListByTypeCode(com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getSystemDicTypeItemListByTypeCodeMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse> getSystemDicTypeItemListByTypeCodeAsync(com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getSystemDicTypeItemListByTypeCodeAsyncMethod, request);
        }

            /**
         * <pre>
         *  获取系统字典
         * </pre>
         */
        @Override
        public void getSystemDicTypeItemListByTypeCode(com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest request, StreamObserver<com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getSystemDicTypeItemListByTypeCodeMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  清除缓存
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.InvalidateCacheResponse invalidateCache(com.ywt.gapi.system.InvalidateCacheRequest request){
            return StubInvocationUtil.unaryCall(invoker, invalidateCacheMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.InvalidateCacheResponse> invalidateCacheAsync(com.ywt.gapi.system.InvalidateCacheRequest request){
            return StubInvocationUtil.unaryCall(invoker, invalidateCacheAsyncMethod, request);
        }

            /**
         * <pre>
         *  清除缓存
         * </pre>
         */
        @Override
        public void invalidateCache(com.ywt.gapi.system.InvalidateCacheRequest request, StreamObserver<com.ywt.gapi.system.InvalidateCacheResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, invalidateCacheMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  获取缓存 keys
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetCacheKeysResponse getCacheKeys(com.ywt.gapi.system.GetCacheKeysRequest request){
            return StubInvocationUtil.unaryCall(invoker, getCacheKeysMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetCacheKeysResponse> getCacheKeysAsync(com.ywt.gapi.system.GetCacheKeysRequest request){
            return StubInvocationUtil.unaryCall(invoker, getCacheKeysAsyncMethod, request);
        }

            /**
         * <pre>
         *  获取缓存 keys
         * </pre>
         */
        @Override
        public void getCacheKeys(com.ywt.gapi.system.GetCacheKeysRequest request, StreamObserver<com.ywt.gapi.system.GetCacheKeysResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getCacheKeysMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  获取配置的内容
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetConfigByTypeResponse getConfigByType(com.ywt.gapi.system.GetConfigByTypeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getConfigByTypeMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetConfigByTypeResponse> getConfigByTypeAsync(com.ywt.gapi.system.GetConfigByTypeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getConfigByTypeAsyncMethod, request);
        }

            /**
         * <pre>
         *  获取配置的内容
         * </pre>
         */
        @Override
        public void getConfigByType(com.ywt.gapi.system.GetConfigByTypeRequest request, StreamObserver<com.ywt.gapi.system.GetConfigByTypeResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getConfigByTypeMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  根据医院IDs得到医院信息		
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetHospitalByHospitalIdsResponse getHospitalByHospitalIds(com.ywt.gapi.system.GetHospitalByHospitalIdsRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalByHospitalIdsMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetHospitalByHospitalIdsResponse> getHospitalByHospitalIdsAsync(com.ywt.gapi.system.GetHospitalByHospitalIdsRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalByHospitalIdsAsyncMethod, request);
        }

            /**
         * <pre>
         *  根据医院IDs得到医院信息		
         * </pre>
         */
        @Override
        public void getHospitalByHospitalIds(com.ywt.gapi.system.GetHospitalByHospitalIdsRequest request, StreamObserver<com.ywt.gapi.system.GetHospitalByHospitalIdsResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getHospitalByHospitalIdsMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  保存GPS记录
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.SaveGPSRecordResponse saveGPSRecord(com.ywt.gapi.system.SaveGPSRecordRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveGPSRecordMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.SaveGPSRecordResponse> saveGPSRecordAsync(com.ywt.gapi.system.SaveGPSRecordRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveGPSRecordAsyncMethod, request);
        }

            /**
         * <pre>
         *  保存GPS记录
         * </pre>
         */
        @Override
        public void saveGPSRecord(com.ywt.gapi.system.SaveGPSRecordRequest request, StreamObserver<com.ywt.gapi.system.SaveGPSRecordResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveGPSRecordMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  根据配置ID获取配置信息
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GetConfigByIdResponse getConfigById(com.ywt.gapi.system.GetConfigByIdRequest request){
            return StubInvocationUtil.unaryCall(invoker, getConfigByIdMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GetConfigByIdResponse> getConfigByIdAsync(com.ywt.gapi.system.GetConfigByIdRequest request){
            return StubInvocationUtil.unaryCall(invoker, getConfigByIdAsyncMethod, request);
        }

            /**
         * <pre>
         *  根据配置ID获取配置信息
         * </pre>
         */
        @Override
        public void getConfigById(com.ywt.gapi.system.GetConfigByIdRequest request, StreamObserver<com.ywt.gapi.system.GetConfigByIdResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getConfigByIdMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  根据名称获取科室列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.SearchDeptListByNameResponse searchDeptListByName(com.ywt.gapi.system.SearchDeptListByNameRequest request){
            return StubInvocationUtil.unaryCall(invoker, searchDeptListByNameMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.SearchDeptListByNameResponse> searchDeptListByNameAsync(com.ywt.gapi.system.SearchDeptListByNameRequest request){
            return StubInvocationUtil.unaryCall(invoker, searchDeptListByNameAsyncMethod, request);
        }

            /**
         * <pre>
         *  根据名称获取科室列表
         * </pre>
         */
        @Override
        public void searchDeptListByName(com.ywt.gapi.system.SearchDeptListByNameRequest request, StreamObserver<com.ywt.gapi.system.SearchDeptListByNameResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, searchDeptListByNameMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  根据名称获取医生列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.SearchDoctorListByNameResponse searchDoctorListByName(com.ywt.gapi.system.SearchDoctorListByNameRequest request){
            return StubInvocationUtil.unaryCall(invoker, searchDoctorListByNameMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.SearchDoctorListByNameResponse> searchDoctorListByNameAsync(com.ywt.gapi.system.SearchDoctorListByNameRequest request){
            return StubInvocationUtil.unaryCall(invoker, searchDoctorListByNameAsyncMethod, request);
        }

            /**
         * <pre>
         *  根据名称获取医生列表
         * </pre>
         */
        @Override
        public void searchDoctorListByName(com.ywt.gapi.system.SearchDoctorListByNameRequest request, StreamObserver<com.ywt.gapi.system.SearchDoctorListByNameResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, searchDoctorListByNameMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  根据姓名从多个医院查找医生
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.SearchDoctorListByNameResponse searchDoctorByNameFromMultiHosp(com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest request){
            return StubInvocationUtil.unaryCall(invoker, searchDoctorByNameFromMultiHospMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.SearchDoctorListByNameResponse> searchDoctorByNameFromMultiHospAsync(com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest request){
            return StubInvocationUtil.unaryCall(invoker, searchDoctorByNameFromMultiHospAsyncMethod, request);
        }

            /**
         * <pre>
         *  根据姓名从多个医院查找医生
         * </pre>
         */
        @Override
        public void searchDoctorByNameFromMultiHosp(com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest request, StreamObserver<com.ywt.gapi.system.SearchDoctorListByNameResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, searchDoctorByNameFromMultiHospMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  得到预约挂号公告信息
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.RetRegisteredAnnouncementResponse getRegisteredAnnouncement(com.ywt.gapi.system.RetRegisteredAnnouncementRequest request){
            return StubInvocationUtil.unaryCall(invoker, getRegisteredAnnouncementMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.RetRegisteredAnnouncementResponse> getRegisteredAnnouncementAsync(com.ywt.gapi.system.RetRegisteredAnnouncementRequest request){
            return StubInvocationUtil.unaryCall(invoker, getRegisteredAnnouncementAsyncMethod, request);
        }

            /**
         * <pre>
         *  得到预约挂号公告信息
         * </pre>
         */
        @Override
        public void getRegisteredAnnouncement(com.ywt.gapi.system.RetRegisteredAnnouncementRequest request, StreamObserver<com.ywt.gapi.system.RetRegisteredAnnouncementResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getRegisteredAnnouncementMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  判断是否有救护车的权限
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result checkHasAmbulanceRight(com.ywt.gapi.system.CheckHasAmbulanceRightRequest request){
            return StubInvocationUtil.unaryCall(invoker, checkHasAmbulanceRightMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> checkHasAmbulanceRightAsync(com.ywt.gapi.system.CheckHasAmbulanceRightRequest request){
            return StubInvocationUtil.unaryCall(invoker, checkHasAmbulanceRightAsyncMethod, request);
        }

            /**
         * <pre>
         *  判断是否有救护车的权限
         * </pre>
         */
        @Override
        public void checkHasAmbulanceRight(com.ywt.gapi.system.CheckHasAmbulanceRightRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, checkHasAmbulanceRightMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  记录救护车位置
         * </pre>
         */
        @Override
        public com.ywt.gapi.Result saveAmbulanceLocation(com.ywt.gapi.system.SaveAmbulanceLocationRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveAmbulanceLocationMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.Result> saveAmbulanceLocationAsync(com.ywt.gapi.system.SaveAmbulanceLocationRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveAmbulanceLocationAsyncMethod, request);
        }

            /**
         * <pre>
         *  记录救护车位置
         * </pre>
         */
        @Override
        public void saveAmbulanceLocation(com.ywt.gapi.system.SaveAmbulanceLocationRequest request, StreamObserver<com.ywt.gapi.Result> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveAmbulanceLocationMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 须知展示
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.CommonAnnouncementResponse showCommonAnnouncement(com.ywt.gapi.system.CommonAnnouncementRequest request){
            return StubInvocationUtil.unaryCall(invoker, showCommonAnnouncementMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.CommonAnnouncementResponse> showCommonAnnouncementAsync(com.ywt.gapi.system.CommonAnnouncementRequest request){
            return StubInvocationUtil.unaryCall(invoker, showCommonAnnouncementAsyncMethod, request);
        }

            /**
         * <pre>
         * 须知展示
         * </pre>
         */
        @Override
        public void showCommonAnnouncement(com.ywt.gapi.system.CommonAnnouncementRequest request, StreamObserver<com.ywt.gapi.system.CommonAnnouncementResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, showCommonAnnouncementMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 就诊指南
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.GuideListResponse getGuideList(com.ywt.gapi.system.GuideListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getGuideListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.GuideListResponse> getGuideListAsync(com.ywt.gapi.system.GuideListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getGuideListAsyncMethod, request);
        }

            /**
         * <pre>
         * 就诊指南
         * </pre>
         */
        @Override
        public void getGuideList(com.ywt.gapi.system.GuideListRequest request, StreamObserver<com.ywt.gapi.system.GuideListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getGuideListMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  根据条件获取医院列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.AreaHospitalListResponse getHospitalListByType(com.ywt.gapi.system.GetHospitalListByTypeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalListByTypeMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.AreaHospitalListResponse> getHospitalListByTypeAsync(com.ywt.gapi.system.GetHospitalListByTypeRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalListByTypeAsyncMethod, request);
        }

            /**
         * <pre>
         *  根据条件获取医院列表
         * </pre>
         */
        @Override
        public void getHospitalListByType(com.ywt.gapi.system.GetHospitalListByTypeRequest request, StreamObserver<com.ywt.gapi.system.AreaHospitalListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getHospitalListByTypeMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 科室介绍
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.IntroduceResponse getIntroduce(com.ywt.gapi.system.IntroduceRequest request){
            return StubInvocationUtil.unaryCall(invoker, getIntroduceMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.IntroduceResponse> getIntroduceAsync(com.ywt.gapi.system.IntroduceRequest request){
            return StubInvocationUtil.unaryCall(invoker, getIntroduceAsyncMethod, request);
        }

            /**
         * <pre>
         * 科室介绍
         * </pre>
         */
        @Override
        public void getIntroduce(com.ywt.gapi.system.IntroduceRequest request, StreamObserver<com.ywt.gapi.system.IntroduceResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getIntroduceMethod , request, responseObserver);
        }
            /**
         * <pre>
         * 住院科室列表
         * </pre>
         */
        @Override
        public com.ywt.gapi.system.HospitalMedicalDeptListResponse hospitalMedicalDeptList(com.ywt.gapi.system.HospitalMedicalDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, hospitalMedicalDeptListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.HospitalMedicalDeptListResponse> hospitalMedicalDeptListAsync(com.ywt.gapi.system.HospitalMedicalDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, hospitalMedicalDeptListAsyncMethod, request);
        }

            /**
         * <pre>
         * 住院科室列表
         * </pre>
         */
        @Override
        public void hospitalMedicalDeptList(com.ywt.gapi.system.HospitalMedicalDeptListRequest request, StreamObserver<com.ywt.gapi.system.HospitalMedicalDeptListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, hospitalMedicalDeptListMethod , request, responseObserver);
        }



    }

    public static abstract class SystemServiceImplBase implements SystemService, ServerService<SystemService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<SystemService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDeptList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDeptListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalDeptList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalDeptListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/checkAppVersion" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/checkAppVersionAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAreaList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAreaListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAddressList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAddressListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveAddress" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveAddressAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/deleteAddress" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/deleteAddressAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAddress" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAddressAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAreaTreeNodeList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAreaTreeNodeListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAppAreaList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAppAreaListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAreaHospitalList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAreaHospitalListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getWeChatConfig" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getWeChatConfigAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDeptByYiDaCode" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDeptByYiDaCodeAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getYiDaDoctorCode" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getYiDaDoctorCodeAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getIcdCodeList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getIcdCodeListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAllAreaList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getAllAreaListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDoctorAddressList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDoctorAddressListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/doctorSaveAddress" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/doctorSaveAddressAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/settingDefaultAddress" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/settingDefaultAddressAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getMsgTplListByModule" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getMsgTplListByModuleAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDoctorDefaultAddress" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDoctorDefaultAddressAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/synchronizingInformation" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/synchronizingInformationAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospital" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/syncTaiheInspectionReport" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/syncTaiheInspectionReportAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/syncTaiheLaboratoryReport" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/syncTaiheLaboratoryReportAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/createHisAddRegisteredFlag" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/createHisAddRegisteredFlagAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getTeamDeptList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getTeamDeptListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/findHospitalList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/findHospitalListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getGeneralAttentionHospitalList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getGeneralAttentionHospitalListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveMiniProgramLog" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveMiniProgramLogAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDept" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getDeptAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getCloudPlatformHospitalList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getCloudPlatformHospitalListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getSystemDicTypeItemListByTypeCode" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getSystemDicTypeItemListByTypeCodeAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/invalidateCache" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/invalidateCacheAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getCacheKeys" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getCacheKeysAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getConfigByType" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getConfigByTypeAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalByHospitalIds" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalByHospitalIdsAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveGPSRecord" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveGPSRecordAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getConfigById" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getConfigByIdAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/searchDeptListByName" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/searchDeptListByNameAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/searchDoctorListByName" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/searchDoctorListByNameAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/searchDoctorByNameFromMultiHosp" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/searchDoctorByNameFromMultiHospAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getRegisteredAnnouncement" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getRegisteredAnnouncementAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/checkHasAmbulanceRight" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/checkHasAmbulanceRightAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveAmbulanceLocation" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveAmbulanceLocationAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/showCommonAnnouncement" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/showCommonAnnouncementAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getGuideList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getGuideListAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalListByType" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalListByTypeAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getIntroduce" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getIntroduceAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/hospitalMedicalDeptList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/hospitalMedicalDeptListAsync" );

            BiConsumer<com.ywt.gapi.system.HospitalListRequest, StreamObserver<com.ywt.gapi.system.HospitalListResponse>> getHospitalListFunc = this::getHospitalList;
            handlers.put(getHospitalListMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalListFunc));
            BiConsumer<com.ywt.gapi.system.HospitalListRequest, StreamObserver<com.ywt.gapi.system.HospitalListResponse>> getHospitalListAsyncFunc = syncToAsync(this::getHospitalList);
            handlers.put(getHospitalListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.DeptListRequest, StreamObserver<com.ywt.gapi.system.DeptListResponse>> getDeptListFunc = this::getDeptList;
            handlers.put(getDeptListMethod.getMethodName(), new UnaryStubMethodHandler<>(getDeptListFunc));
            BiConsumer<com.ywt.gapi.system.DeptListRequest, StreamObserver<com.ywt.gapi.system.DeptListResponse>> getDeptListAsyncFunc = syncToAsync(this::getDeptList);
            handlers.put(getDeptListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getDeptListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.HospitalDeptListRequest, StreamObserver<com.ywt.gapi.system.HospitalDeptListResponse>> getHospitalDeptListFunc = this::getHospitalDeptList;
            handlers.put(getHospitalDeptListMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalDeptListFunc));
            BiConsumer<com.ywt.gapi.system.HospitalDeptListRequest, StreamObserver<com.ywt.gapi.system.HospitalDeptListResponse>> getHospitalDeptListAsyncFunc = syncToAsync(this::getHospitalDeptList);
            handlers.put(getHospitalDeptListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalDeptListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.CheckAppVersionRequest, StreamObserver<com.ywt.gapi.system.CheckAppVersionResponse>> checkAppVersionFunc = this::checkAppVersion;
            handlers.put(checkAppVersionMethod.getMethodName(), new UnaryStubMethodHandler<>(checkAppVersionFunc));
            BiConsumer<com.ywt.gapi.system.CheckAppVersionRequest, StreamObserver<com.ywt.gapi.system.CheckAppVersionResponse>> checkAppVersionAsyncFunc = syncToAsync(this::checkAppVersion);
            handlers.put(checkAppVersionProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(checkAppVersionAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetAreaListRequest, StreamObserver<com.ywt.gapi.system.GetAreaListResponse>> getAreaListFunc = this::getAreaList;
            handlers.put(getAreaListMethod.getMethodName(), new UnaryStubMethodHandler<>(getAreaListFunc));
            BiConsumer<com.ywt.gapi.system.GetAreaListRequest, StreamObserver<com.ywt.gapi.system.GetAreaListResponse>> getAreaListAsyncFunc = syncToAsync(this::getAreaList);
            handlers.put(getAreaListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAreaListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetAddressListRequest, StreamObserver<com.ywt.gapi.system.GetAddressListResponse>> getAddressListFunc = this::getAddressList;
            handlers.put(getAddressListMethod.getMethodName(), new UnaryStubMethodHandler<>(getAddressListFunc));
            BiConsumer<com.ywt.gapi.system.GetAddressListRequest, StreamObserver<com.ywt.gapi.system.GetAddressListResponse>> getAddressListAsyncFunc = syncToAsync(this::getAddressList);
            handlers.put(getAddressListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAddressListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SaveAddressRequest, StreamObserver<com.ywt.gapi.system.SaveAddressResponse>> saveAddressFunc = this::saveAddress;
            handlers.put(saveAddressMethod.getMethodName(), new UnaryStubMethodHandler<>(saveAddressFunc));
            BiConsumer<com.ywt.gapi.system.SaveAddressRequest, StreamObserver<com.ywt.gapi.system.SaveAddressResponse>> saveAddressAsyncFunc = syncToAsync(this::saveAddress);
            handlers.put(saveAddressProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveAddressAsyncFunc));
            BiConsumer<com.ywt.gapi.system.DeleteAddressRequest, StreamObserver<com.ywt.gapi.Result>> deleteAddressFunc = this::deleteAddress;
            handlers.put(deleteAddressMethod.getMethodName(), new UnaryStubMethodHandler<>(deleteAddressFunc));
            BiConsumer<com.ywt.gapi.system.DeleteAddressRequest, StreamObserver<com.ywt.gapi.Result>> deleteAddressAsyncFunc = syncToAsync(this::deleteAddress);
            handlers.put(deleteAddressProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(deleteAddressAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetAddressRequest, StreamObserver<com.ywt.gapi.system.GetAddressResponse>> getAddressFunc = this::getAddress;
            handlers.put(getAddressMethod.getMethodName(), new UnaryStubMethodHandler<>(getAddressFunc));
            BiConsumer<com.ywt.gapi.system.GetAddressRequest, StreamObserver<com.ywt.gapi.system.GetAddressResponse>> getAddressAsyncFunc = syncToAsync(this::getAddress);
            handlers.put(getAddressProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAddressAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetAreaTreeNodeListRequest, StreamObserver<com.ywt.gapi.system.GetAreaTreeNodeListResponse>> getAreaTreeNodeListFunc = this::getAreaTreeNodeList;
            handlers.put(getAreaTreeNodeListMethod.getMethodName(), new UnaryStubMethodHandler<>(getAreaTreeNodeListFunc));
            BiConsumer<com.ywt.gapi.system.GetAreaTreeNodeListRequest, StreamObserver<com.ywt.gapi.system.GetAreaTreeNodeListResponse>> getAreaTreeNodeListAsyncFunc = syncToAsync(this::getAreaTreeNodeList);
            handlers.put(getAreaTreeNodeListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAreaTreeNodeListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.AreaListRequest, StreamObserver<com.ywt.gapi.system.AreaListResponse>> getAppAreaListFunc = this::getAppAreaList;
            handlers.put(getAppAreaListMethod.getMethodName(), new UnaryStubMethodHandler<>(getAppAreaListFunc));
            BiConsumer<com.ywt.gapi.system.AreaListRequest, StreamObserver<com.ywt.gapi.system.AreaListResponse>> getAppAreaListAsyncFunc = syncToAsync(this::getAppAreaList);
            handlers.put(getAppAreaListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAppAreaListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.AreaHospitalListRequest, StreamObserver<com.ywt.gapi.system.AreaHospitalListResponse>> getAreaHospitalListFunc = this::getAreaHospitalList;
            handlers.put(getAreaHospitalListMethod.getMethodName(), new UnaryStubMethodHandler<>(getAreaHospitalListFunc));
            BiConsumer<com.ywt.gapi.system.AreaHospitalListRequest, StreamObserver<com.ywt.gapi.system.AreaHospitalListResponse>> getAreaHospitalListAsyncFunc = syncToAsync(this::getAreaHospitalList);
            handlers.put(getAreaHospitalListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAreaHospitalListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetWeChatConfigRequest, StreamObserver<com.ywt.gapi.system.GetWeChatConfigResponse>> getWeChatConfigFunc = this::getWeChatConfig;
            handlers.put(getWeChatConfigMethod.getMethodName(), new UnaryStubMethodHandler<>(getWeChatConfigFunc));
            BiConsumer<com.ywt.gapi.system.GetWeChatConfigRequest, StreamObserver<com.ywt.gapi.system.GetWeChatConfigResponse>> getWeChatConfigAsyncFunc = syncToAsync(this::getWeChatConfig);
            handlers.put(getWeChatConfigProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getWeChatConfigAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetDeptByYiDaCodeRequest, StreamObserver<com.ywt.gapi.system.GetDeptByYiDaCodeResponse>> getDeptByYiDaCodeFunc = this::getDeptByYiDaCode;
            handlers.put(getDeptByYiDaCodeMethod.getMethodName(), new UnaryStubMethodHandler<>(getDeptByYiDaCodeFunc));
            BiConsumer<com.ywt.gapi.system.GetDeptByYiDaCodeRequest, StreamObserver<com.ywt.gapi.system.GetDeptByYiDaCodeResponse>> getDeptByYiDaCodeAsyncFunc = syncToAsync(this::getDeptByYiDaCode);
            handlers.put(getDeptByYiDaCodeProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getDeptByYiDaCodeAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetYiDaDoctorCodeRequest, StreamObserver<com.ywt.gapi.system.GetYiDaDoctorCodeResponse>> getYiDaDoctorCodeFunc = this::getYiDaDoctorCode;
            handlers.put(getYiDaDoctorCodeMethod.getMethodName(), new UnaryStubMethodHandler<>(getYiDaDoctorCodeFunc));
            BiConsumer<com.ywt.gapi.system.GetYiDaDoctorCodeRequest, StreamObserver<com.ywt.gapi.system.GetYiDaDoctorCodeResponse>> getYiDaDoctorCodeAsyncFunc = syncToAsync(this::getYiDaDoctorCode);
            handlers.put(getYiDaDoctorCodeProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getYiDaDoctorCodeAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetIcdCodeListRequest, StreamObserver<com.ywt.gapi.system.GetIcdCodeListResponse>> getIcdCodeListFunc = this::getIcdCodeList;
            handlers.put(getIcdCodeListMethod.getMethodName(), new UnaryStubMethodHandler<>(getIcdCodeListFunc));
            BiConsumer<com.ywt.gapi.system.GetIcdCodeListRequest, StreamObserver<com.ywt.gapi.system.GetIcdCodeListResponse>> getIcdCodeListAsyncFunc = syncToAsync(this::getIcdCodeList);
            handlers.put(getIcdCodeListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getIcdCodeListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetAreaListRequest, StreamObserver<com.ywt.gapi.system.GetAreaListResponse>> getAllAreaListFunc = this::getAllAreaList;
            handlers.put(getAllAreaListMethod.getMethodName(), new UnaryStubMethodHandler<>(getAllAreaListFunc));
            BiConsumer<com.ywt.gapi.system.GetAreaListRequest, StreamObserver<com.ywt.gapi.system.GetAreaListResponse>> getAllAreaListAsyncFunc = syncToAsync(this::getAllAreaList);
            handlers.put(getAllAreaListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getAllAreaListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetAddressListRequest, StreamObserver<com.ywt.gapi.system.GetAddressListResponse>> getDoctorAddressListFunc = this::getDoctorAddressList;
            handlers.put(getDoctorAddressListMethod.getMethodName(), new UnaryStubMethodHandler<>(getDoctorAddressListFunc));
            BiConsumer<com.ywt.gapi.system.GetAddressListRequest, StreamObserver<com.ywt.gapi.system.GetAddressListResponse>> getDoctorAddressListAsyncFunc = syncToAsync(this::getDoctorAddressList);
            handlers.put(getDoctorAddressListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getDoctorAddressListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SaveAddressRequest, StreamObserver<com.ywt.gapi.Result>> doctorSaveAddressFunc = this::doctorSaveAddress;
            handlers.put(doctorSaveAddressMethod.getMethodName(), new UnaryStubMethodHandler<>(doctorSaveAddressFunc));
            BiConsumer<com.ywt.gapi.system.SaveAddressRequest, StreamObserver<com.ywt.gapi.Result>> doctorSaveAddressAsyncFunc = syncToAsync(this::doctorSaveAddress);
            handlers.put(doctorSaveAddressProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(doctorSaveAddressAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SettingDefaultAddressRequest, StreamObserver<com.ywt.gapi.Result>> settingDefaultAddressFunc = this::settingDefaultAddress;
            handlers.put(settingDefaultAddressMethod.getMethodName(), new UnaryStubMethodHandler<>(settingDefaultAddressFunc));
            BiConsumer<com.ywt.gapi.system.SettingDefaultAddressRequest, StreamObserver<com.ywt.gapi.Result>> settingDefaultAddressAsyncFunc = syncToAsync(this::settingDefaultAddress);
            handlers.put(settingDefaultAddressProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(settingDefaultAddressAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetMsgTplListByModuleRequest, StreamObserver<com.ywt.gapi.system.GetMsgTplListByModuleResponse>> getMsgTplListByModuleFunc = this::getMsgTplListByModule;
            handlers.put(getMsgTplListByModuleMethod.getMethodName(), new UnaryStubMethodHandler<>(getMsgTplListByModuleFunc));
            BiConsumer<com.ywt.gapi.system.GetMsgTplListByModuleRequest, StreamObserver<com.ywt.gapi.system.GetMsgTplListByModuleResponse>> getMsgTplListByModuleAsyncFunc = syncToAsync(this::getMsgTplListByModule);
            handlers.put(getMsgTplListByModuleProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getMsgTplListByModuleAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetDoctorAddressRequest, StreamObserver<com.ywt.gapi.system.GetDoctorDefaultAddressResponse>> getDoctorDefaultAddressFunc = this::getDoctorDefaultAddress;
            handlers.put(getDoctorDefaultAddressMethod.getMethodName(), new UnaryStubMethodHandler<>(getDoctorDefaultAddressFunc));
            BiConsumer<com.ywt.gapi.system.GetDoctorAddressRequest, StreamObserver<com.ywt.gapi.system.GetDoctorDefaultAddressResponse>> getDoctorDefaultAddressAsyncFunc = syncToAsync(this::getDoctorDefaultAddress);
            handlers.put(getDoctorDefaultAddressProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getDoctorDefaultAddressAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SynchronizingInformationRequest, StreamObserver<com.ywt.gapi.Result>> synchronizingInformationFunc = this::synchronizingInformation;
            handlers.put(synchronizingInformationMethod.getMethodName(), new UnaryStubMethodHandler<>(synchronizingInformationFunc));
            BiConsumer<com.ywt.gapi.system.SynchronizingInformationRequest, StreamObserver<com.ywt.gapi.Result>> synchronizingInformationAsyncFunc = syncToAsync(this::synchronizingInformation);
            handlers.put(synchronizingInformationProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(synchronizingInformationAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalRequest, StreamObserver<com.ywt.gapi.system.GetHospitalResponse>> getHospitalFunc = this::getHospital;
            handlers.put(getHospitalMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalRequest, StreamObserver<com.ywt.gapi.system.GetHospitalResponse>> getHospitalAsyncFunc = syncToAsync(this::getHospital);
            handlers.put(getHospitalProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SyncTaiheInspectionReportRequest, StreamObserver<com.ywt.gapi.Result>> syncTaiheInspectionReportFunc = this::syncTaiheInspectionReport;
            handlers.put(syncTaiheInspectionReportMethod.getMethodName(), new UnaryStubMethodHandler<>(syncTaiheInspectionReportFunc));
            BiConsumer<com.ywt.gapi.system.SyncTaiheInspectionReportRequest, StreamObserver<com.ywt.gapi.Result>> syncTaiheInspectionReportAsyncFunc = syncToAsync(this::syncTaiheInspectionReport);
            handlers.put(syncTaiheInspectionReportProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(syncTaiheInspectionReportAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest, StreamObserver<com.ywt.gapi.Result>> syncTaiheLaboratoryReportFunc = this::syncTaiheLaboratoryReport;
            handlers.put(syncTaiheLaboratoryReportMethod.getMethodName(), new UnaryStubMethodHandler<>(syncTaiheLaboratoryReportFunc));
            BiConsumer<com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest, StreamObserver<com.ywt.gapi.Result>> syncTaiheLaboratoryReportAsyncFunc = syncToAsync(this::syncTaiheLaboratoryReport);
            handlers.put(syncTaiheLaboratoryReportProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(syncTaiheLaboratoryReportAsyncFunc));
            BiConsumer<com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest, StreamObserver<com.ywt.gapi.Result>> createHisAddRegisteredFlagFunc = this::createHisAddRegisteredFlag;
            handlers.put(createHisAddRegisteredFlagMethod.getMethodName(), new UnaryStubMethodHandler<>(createHisAddRegisteredFlagFunc));
            BiConsumer<com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest, StreamObserver<com.ywt.gapi.Result>> createHisAddRegisteredFlagAsyncFunc = syncToAsync(this::createHisAddRegisteredFlag);
            handlers.put(createHisAddRegisteredFlagProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(createHisAddRegisteredFlagAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetTeamDeptListRequest, StreamObserver<com.ywt.gapi.system.GetTeamDeptListResponse>> getTeamDeptListFunc = this::getTeamDeptList;
            handlers.put(getTeamDeptListMethod.getMethodName(), new UnaryStubMethodHandler<>(getTeamDeptListFunc));
            BiConsumer<com.ywt.gapi.system.GetTeamDeptListRequest, StreamObserver<com.ywt.gapi.system.GetTeamDeptListResponse>> getTeamDeptListAsyncFunc = syncToAsync(this::getTeamDeptList);
            handlers.put(getTeamDeptListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getTeamDeptListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalListRequest, StreamObserver<com.ywt.gapi.system.HospitalListResponse>> findHospitalListFunc = this::findHospitalList;
            handlers.put(findHospitalListMethod.getMethodName(), new UnaryStubMethodHandler<>(findHospitalListFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalListRequest, StreamObserver<com.ywt.gapi.system.HospitalListResponse>> findHospitalListAsyncFunc = syncToAsync(this::findHospitalList);
            handlers.put(findHospitalListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(findHospitalListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalListRequest, StreamObserver<com.ywt.gapi.system.HospitalListResponse>> getGeneralAttentionHospitalListFunc = this::getGeneralAttentionHospitalList;
            handlers.put(getGeneralAttentionHospitalListMethod.getMethodName(), new UnaryStubMethodHandler<>(getGeneralAttentionHospitalListFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalListRequest, StreamObserver<com.ywt.gapi.system.HospitalListResponse>> getGeneralAttentionHospitalListAsyncFunc = syncToAsync(this::getGeneralAttentionHospitalList);
            handlers.put(getGeneralAttentionHospitalListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getGeneralAttentionHospitalListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SaveMiniProgramLogRequest, StreamObserver<com.ywt.gapi.system.SaveMiniProgramLogResponse>> saveMiniProgramLogFunc = this::saveMiniProgramLog;
            handlers.put(saveMiniProgramLogMethod.getMethodName(), new UnaryStubMethodHandler<>(saveMiniProgramLogFunc));
            BiConsumer<com.ywt.gapi.system.SaveMiniProgramLogRequest, StreamObserver<com.ywt.gapi.system.SaveMiniProgramLogResponse>> saveMiniProgramLogAsyncFunc = syncToAsync(this::saveMiniProgramLog);
            handlers.put(saveMiniProgramLogProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveMiniProgramLogAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetDeptRequest, StreamObserver<com.ywt.gapi.system.GetDeptResponse>> getDeptFunc = this::getDept;
            handlers.put(getDeptMethod.getMethodName(), new UnaryStubMethodHandler<>(getDeptFunc));
            BiConsumer<com.ywt.gapi.system.GetDeptRequest, StreamObserver<com.ywt.gapi.system.GetDeptResponse>> getDeptAsyncFunc = syncToAsync(this::getDept);
            handlers.put(getDeptProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getDeptAsyncFunc));
            BiConsumer<com.ywt.gapi.system.HospitalListRequest, StreamObserver<com.ywt.gapi.system.CloudPlatformHospitalListResponse>> getCloudPlatformHospitalListFunc = this::getCloudPlatformHospitalList;
            handlers.put(getCloudPlatformHospitalListMethod.getMethodName(), new UnaryStubMethodHandler<>(getCloudPlatformHospitalListFunc));
            BiConsumer<com.ywt.gapi.system.HospitalListRequest, StreamObserver<com.ywt.gapi.system.CloudPlatformHospitalListResponse>> getCloudPlatformHospitalListAsyncFunc = syncToAsync(this::getCloudPlatformHospitalList);
            handlers.put(getCloudPlatformHospitalListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getCloudPlatformHospitalListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest, StreamObserver<com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse>> getSystemDicTypeItemListByTypeCodeFunc = this::getSystemDicTypeItemListByTypeCode;
            handlers.put(getSystemDicTypeItemListByTypeCodeMethod.getMethodName(), new UnaryStubMethodHandler<>(getSystemDicTypeItemListByTypeCodeFunc));
            BiConsumer<com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest, StreamObserver<com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse>> getSystemDicTypeItemListByTypeCodeAsyncFunc = syncToAsync(this::getSystemDicTypeItemListByTypeCode);
            handlers.put(getSystemDicTypeItemListByTypeCodeProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getSystemDicTypeItemListByTypeCodeAsyncFunc));
            BiConsumer<com.ywt.gapi.system.InvalidateCacheRequest, StreamObserver<com.ywt.gapi.system.InvalidateCacheResponse>> invalidateCacheFunc = this::invalidateCache;
            handlers.put(invalidateCacheMethod.getMethodName(), new UnaryStubMethodHandler<>(invalidateCacheFunc));
            BiConsumer<com.ywt.gapi.system.InvalidateCacheRequest, StreamObserver<com.ywt.gapi.system.InvalidateCacheResponse>> invalidateCacheAsyncFunc = syncToAsync(this::invalidateCache);
            handlers.put(invalidateCacheProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(invalidateCacheAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetCacheKeysRequest, StreamObserver<com.ywt.gapi.system.GetCacheKeysResponse>> getCacheKeysFunc = this::getCacheKeys;
            handlers.put(getCacheKeysMethod.getMethodName(), new UnaryStubMethodHandler<>(getCacheKeysFunc));
            BiConsumer<com.ywt.gapi.system.GetCacheKeysRequest, StreamObserver<com.ywt.gapi.system.GetCacheKeysResponse>> getCacheKeysAsyncFunc = syncToAsync(this::getCacheKeys);
            handlers.put(getCacheKeysProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getCacheKeysAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetConfigByTypeRequest, StreamObserver<com.ywt.gapi.system.GetConfigByTypeResponse>> getConfigByTypeFunc = this::getConfigByType;
            handlers.put(getConfigByTypeMethod.getMethodName(), new UnaryStubMethodHandler<>(getConfigByTypeFunc));
            BiConsumer<com.ywt.gapi.system.GetConfigByTypeRequest, StreamObserver<com.ywt.gapi.system.GetConfigByTypeResponse>> getConfigByTypeAsyncFunc = syncToAsync(this::getConfigByType);
            handlers.put(getConfigByTypeProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getConfigByTypeAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalByHospitalIdsRequest, StreamObserver<com.ywt.gapi.system.GetHospitalByHospitalIdsResponse>> getHospitalByHospitalIdsFunc = this::getHospitalByHospitalIds;
            handlers.put(getHospitalByHospitalIdsMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalByHospitalIdsFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalByHospitalIdsRequest, StreamObserver<com.ywt.gapi.system.GetHospitalByHospitalIdsResponse>> getHospitalByHospitalIdsAsyncFunc = syncToAsync(this::getHospitalByHospitalIds);
            handlers.put(getHospitalByHospitalIdsProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalByHospitalIdsAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SaveGPSRecordRequest, StreamObserver<com.ywt.gapi.system.SaveGPSRecordResponse>> saveGPSRecordFunc = this::saveGPSRecord;
            handlers.put(saveGPSRecordMethod.getMethodName(), new UnaryStubMethodHandler<>(saveGPSRecordFunc));
            BiConsumer<com.ywt.gapi.system.SaveGPSRecordRequest, StreamObserver<com.ywt.gapi.system.SaveGPSRecordResponse>> saveGPSRecordAsyncFunc = syncToAsync(this::saveGPSRecord);
            handlers.put(saveGPSRecordProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveGPSRecordAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetConfigByIdRequest, StreamObserver<com.ywt.gapi.system.GetConfigByIdResponse>> getConfigByIdFunc = this::getConfigById;
            handlers.put(getConfigByIdMethod.getMethodName(), new UnaryStubMethodHandler<>(getConfigByIdFunc));
            BiConsumer<com.ywt.gapi.system.GetConfigByIdRequest, StreamObserver<com.ywt.gapi.system.GetConfigByIdResponse>> getConfigByIdAsyncFunc = syncToAsync(this::getConfigById);
            handlers.put(getConfigByIdProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getConfigByIdAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SearchDeptListByNameRequest, StreamObserver<com.ywt.gapi.system.SearchDeptListByNameResponse>> searchDeptListByNameFunc = this::searchDeptListByName;
            handlers.put(searchDeptListByNameMethod.getMethodName(), new UnaryStubMethodHandler<>(searchDeptListByNameFunc));
            BiConsumer<com.ywt.gapi.system.SearchDeptListByNameRequest, StreamObserver<com.ywt.gapi.system.SearchDeptListByNameResponse>> searchDeptListByNameAsyncFunc = syncToAsync(this::searchDeptListByName);
            handlers.put(searchDeptListByNameProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(searchDeptListByNameAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SearchDoctorListByNameRequest, StreamObserver<com.ywt.gapi.system.SearchDoctorListByNameResponse>> searchDoctorListByNameFunc = this::searchDoctorListByName;
            handlers.put(searchDoctorListByNameMethod.getMethodName(), new UnaryStubMethodHandler<>(searchDoctorListByNameFunc));
            BiConsumer<com.ywt.gapi.system.SearchDoctorListByNameRequest, StreamObserver<com.ywt.gapi.system.SearchDoctorListByNameResponse>> searchDoctorListByNameAsyncFunc = syncToAsync(this::searchDoctorListByName);
            handlers.put(searchDoctorListByNameProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(searchDoctorListByNameAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest, StreamObserver<com.ywt.gapi.system.SearchDoctorListByNameResponse>> searchDoctorByNameFromMultiHospFunc = this::searchDoctorByNameFromMultiHosp;
            handlers.put(searchDoctorByNameFromMultiHospMethod.getMethodName(), new UnaryStubMethodHandler<>(searchDoctorByNameFromMultiHospFunc));
            BiConsumer<com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest, StreamObserver<com.ywt.gapi.system.SearchDoctorListByNameResponse>> searchDoctorByNameFromMultiHospAsyncFunc = syncToAsync(this::searchDoctorByNameFromMultiHosp);
            handlers.put(searchDoctorByNameFromMultiHospProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(searchDoctorByNameFromMultiHospAsyncFunc));
            BiConsumer<com.ywt.gapi.system.RetRegisteredAnnouncementRequest, StreamObserver<com.ywt.gapi.system.RetRegisteredAnnouncementResponse>> getRegisteredAnnouncementFunc = this::getRegisteredAnnouncement;
            handlers.put(getRegisteredAnnouncementMethod.getMethodName(), new UnaryStubMethodHandler<>(getRegisteredAnnouncementFunc));
            BiConsumer<com.ywt.gapi.system.RetRegisteredAnnouncementRequest, StreamObserver<com.ywt.gapi.system.RetRegisteredAnnouncementResponse>> getRegisteredAnnouncementAsyncFunc = syncToAsync(this::getRegisteredAnnouncement);
            handlers.put(getRegisteredAnnouncementProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getRegisteredAnnouncementAsyncFunc));
            BiConsumer<com.ywt.gapi.system.CheckHasAmbulanceRightRequest, StreamObserver<com.ywt.gapi.Result>> checkHasAmbulanceRightFunc = this::checkHasAmbulanceRight;
            handlers.put(checkHasAmbulanceRightMethod.getMethodName(), new UnaryStubMethodHandler<>(checkHasAmbulanceRightFunc));
            BiConsumer<com.ywt.gapi.system.CheckHasAmbulanceRightRequest, StreamObserver<com.ywt.gapi.Result>> checkHasAmbulanceRightAsyncFunc = syncToAsync(this::checkHasAmbulanceRight);
            handlers.put(checkHasAmbulanceRightProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(checkHasAmbulanceRightAsyncFunc));
            BiConsumer<com.ywt.gapi.system.SaveAmbulanceLocationRequest, StreamObserver<com.ywt.gapi.Result>> saveAmbulanceLocationFunc = this::saveAmbulanceLocation;
            handlers.put(saveAmbulanceLocationMethod.getMethodName(), new UnaryStubMethodHandler<>(saveAmbulanceLocationFunc));
            BiConsumer<com.ywt.gapi.system.SaveAmbulanceLocationRequest, StreamObserver<com.ywt.gapi.Result>> saveAmbulanceLocationAsyncFunc = syncToAsync(this::saveAmbulanceLocation);
            handlers.put(saveAmbulanceLocationProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveAmbulanceLocationAsyncFunc));
            BiConsumer<com.ywt.gapi.system.CommonAnnouncementRequest, StreamObserver<com.ywt.gapi.system.CommonAnnouncementResponse>> showCommonAnnouncementFunc = this::showCommonAnnouncement;
            handlers.put(showCommonAnnouncementMethod.getMethodName(), new UnaryStubMethodHandler<>(showCommonAnnouncementFunc));
            BiConsumer<com.ywt.gapi.system.CommonAnnouncementRequest, StreamObserver<com.ywt.gapi.system.CommonAnnouncementResponse>> showCommonAnnouncementAsyncFunc = syncToAsync(this::showCommonAnnouncement);
            handlers.put(showCommonAnnouncementProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(showCommonAnnouncementAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GuideListRequest, StreamObserver<com.ywt.gapi.system.GuideListResponse>> getGuideListFunc = this::getGuideList;
            handlers.put(getGuideListMethod.getMethodName(), new UnaryStubMethodHandler<>(getGuideListFunc));
            BiConsumer<com.ywt.gapi.system.GuideListRequest, StreamObserver<com.ywt.gapi.system.GuideListResponse>> getGuideListAsyncFunc = syncToAsync(this::getGuideList);
            handlers.put(getGuideListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getGuideListAsyncFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalListByTypeRequest, StreamObserver<com.ywt.gapi.system.AreaHospitalListResponse>> getHospitalListByTypeFunc = this::getHospitalListByType;
            handlers.put(getHospitalListByTypeMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalListByTypeFunc));
            BiConsumer<com.ywt.gapi.system.GetHospitalListByTypeRequest, StreamObserver<com.ywt.gapi.system.AreaHospitalListResponse>> getHospitalListByTypeAsyncFunc = syncToAsync(this::getHospitalListByType);
            handlers.put(getHospitalListByTypeProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalListByTypeAsyncFunc));
            BiConsumer<com.ywt.gapi.system.IntroduceRequest, StreamObserver<com.ywt.gapi.system.IntroduceResponse>> getIntroduceFunc = this::getIntroduce;
            handlers.put(getIntroduceMethod.getMethodName(), new UnaryStubMethodHandler<>(getIntroduceFunc));
            BiConsumer<com.ywt.gapi.system.IntroduceRequest, StreamObserver<com.ywt.gapi.system.IntroduceResponse>> getIntroduceAsyncFunc = syncToAsync(this::getIntroduce);
            handlers.put(getIntroduceProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getIntroduceAsyncFunc));
            BiConsumer<com.ywt.gapi.system.HospitalMedicalDeptListRequest, StreamObserver<com.ywt.gapi.system.HospitalMedicalDeptListResponse>> hospitalMedicalDeptListFunc = this::hospitalMedicalDeptList;
            handlers.put(hospitalMedicalDeptListMethod.getMethodName(), new UnaryStubMethodHandler<>(hospitalMedicalDeptListFunc));
            BiConsumer<com.ywt.gapi.system.HospitalMedicalDeptListRequest, StreamObserver<com.ywt.gapi.system.HospitalMedicalDeptListResponse>> hospitalMedicalDeptListAsyncFunc = syncToAsync(this::hospitalMedicalDeptList);
            handlers.put(hospitalMedicalDeptListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(hospitalMedicalDeptListAsyncFunc));




            return new StubInvoker<>(this, url, SystemService.class, handlers);
        }


        @Override
        public com.ywt.gapi.system.HospitalListResponse getHospitalList(com.ywt.gapi.system.HospitalListRequest request){
            throw unimplementedMethodException(getHospitalListMethod);
        }

        @Override
        public com.ywt.gapi.system.DeptListResponse getDeptList(com.ywt.gapi.system.DeptListRequest request){
            throw unimplementedMethodException(getDeptListMethod);
        }

        @Override
        public com.ywt.gapi.system.HospitalDeptListResponse getHospitalDeptList(com.ywt.gapi.system.HospitalDeptListRequest request){
            throw unimplementedMethodException(getHospitalDeptListMethod);
        }

        @Override
        public com.ywt.gapi.system.CheckAppVersionResponse checkAppVersion(com.ywt.gapi.system.CheckAppVersionRequest request){
            throw unimplementedMethodException(checkAppVersionMethod);
        }

        @Override
        public com.ywt.gapi.system.GetAreaListResponse getAreaList(com.ywt.gapi.system.GetAreaListRequest request){
            throw unimplementedMethodException(getAreaListMethod);
        }

        @Override
        public com.ywt.gapi.system.GetAddressListResponse getAddressList(com.ywt.gapi.system.GetAddressListRequest request){
            throw unimplementedMethodException(getAddressListMethod);
        }

        @Override
        public com.ywt.gapi.system.SaveAddressResponse saveAddress(com.ywt.gapi.system.SaveAddressRequest request){
            throw unimplementedMethodException(saveAddressMethod);
        }

        @Override
        public com.ywt.gapi.Result deleteAddress(com.ywt.gapi.system.DeleteAddressRequest request){
            throw unimplementedMethodException(deleteAddressMethod);
        }

        @Override
        public com.ywt.gapi.system.GetAddressResponse getAddress(com.ywt.gapi.system.GetAddressRequest request){
            throw unimplementedMethodException(getAddressMethod);
        }

        @Override
        public com.ywt.gapi.system.GetAreaTreeNodeListResponse getAreaTreeNodeList(com.ywt.gapi.system.GetAreaTreeNodeListRequest request){
            throw unimplementedMethodException(getAreaTreeNodeListMethod);
        }

        @Override
        public com.ywt.gapi.system.AreaListResponse getAppAreaList(com.ywt.gapi.system.AreaListRequest request){
            throw unimplementedMethodException(getAppAreaListMethod);
        }

        @Override
        public com.ywt.gapi.system.AreaHospitalListResponse getAreaHospitalList(com.ywt.gapi.system.AreaHospitalListRequest request){
            throw unimplementedMethodException(getAreaHospitalListMethod);
        }

        @Override
        public com.ywt.gapi.system.GetWeChatConfigResponse getWeChatConfig(com.ywt.gapi.system.GetWeChatConfigRequest request){
            throw unimplementedMethodException(getWeChatConfigMethod);
        }

        @Override
        public com.ywt.gapi.system.GetDeptByYiDaCodeResponse getDeptByYiDaCode(com.ywt.gapi.system.GetDeptByYiDaCodeRequest request){
            throw unimplementedMethodException(getDeptByYiDaCodeMethod);
        }

        @Override
        public com.ywt.gapi.system.GetYiDaDoctorCodeResponse getYiDaDoctorCode(com.ywt.gapi.system.GetYiDaDoctorCodeRequest request){
            throw unimplementedMethodException(getYiDaDoctorCodeMethod);
        }

        @Override
        public com.ywt.gapi.system.GetIcdCodeListResponse getIcdCodeList(com.ywt.gapi.system.GetIcdCodeListRequest request){
            throw unimplementedMethodException(getIcdCodeListMethod);
        }

        @Override
        public com.ywt.gapi.system.GetAreaListResponse getAllAreaList(com.ywt.gapi.system.GetAreaListRequest request){
            throw unimplementedMethodException(getAllAreaListMethod);
        }

        @Override
        public com.ywt.gapi.system.GetAddressListResponse getDoctorAddressList(com.ywt.gapi.system.GetAddressListRequest request){
            throw unimplementedMethodException(getDoctorAddressListMethod);
        }

        @Override
        public com.ywt.gapi.Result doctorSaveAddress(com.ywt.gapi.system.SaveAddressRequest request){
            throw unimplementedMethodException(doctorSaveAddressMethod);
        }

        @Override
        public com.ywt.gapi.Result settingDefaultAddress(com.ywt.gapi.system.SettingDefaultAddressRequest request){
            throw unimplementedMethodException(settingDefaultAddressMethod);
        }

        @Override
        public com.ywt.gapi.system.GetMsgTplListByModuleResponse getMsgTplListByModule(com.ywt.gapi.system.GetMsgTplListByModuleRequest request){
            throw unimplementedMethodException(getMsgTplListByModuleMethod);
        }

        @Override
        public com.ywt.gapi.system.GetDoctorDefaultAddressResponse getDoctorDefaultAddress(com.ywt.gapi.system.GetDoctorAddressRequest request){
            throw unimplementedMethodException(getDoctorDefaultAddressMethod);
        }

        @Override
        public com.ywt.gapi.Result synchronizingInformation(com.ywt.gapi.system.SynchronizingInformationRequest request){
            throw unimplementedMethodException(synchronizingInformationMethod);
        }

        @Override
        public com.ywt.gapi.system.GetHospitalResponse getHospital(com.ywt.gapi.system.GetHospitalRequest request){
            throw unimplementedMethodException(getHospitalMethod);
        }

        @Override
        public com.ywt.gapi.Result syncTaiheInspectionReport(com.ywt.gapi.system.SyncTaiheInspectionReportRequest request){
            throw unimplementedMethodException(syncTaiheInspectionReportMethod);
        }

        @Override
        public com.ywt.gapi.Result syncTaiheLaboratoryReport(com.ywt.gapi.system.SyncTaiheLaboratoryReportRequest request){
            throw unimplementedMethodException(syncTaiheLaboratoryReportMethod);
        }

        @Override
        public com.ywt.gapi.Result createHisAddRegisteredFlag(com.ywt.gapi.system.CreateHisAddRegisteredFlagRequest request){
            throw unimplementedMethodException(createHisAddRegisteredFlagMethod);
        }

        @Override
        public com.ywt.gapi.system.GetTeamDeptListResponse getTeamDeptList(com.ywt.gapi.system.GetTeamDeptListRequest request){
            throw unimplementedMethodException(getTeamDeptListMethod);
        }

        @Override
        public com.ywt.gapi.system.HospitalListResponse findHospitalList(com.ywt.gapi.system.GetHospitalListRequest request){
            throw unimplementedMethodException(findHospitalListMethod);
        }

        @Override
        public com.ywt.gapi.system.HospitalListResponse getGeneralAttentionHospitalList(com.ywt.gapi.system.GetHospitalListRequest request){
            throw unimplementedMethodException(getGeneralAttentionHospitalListMethod);
        }

        @Override
        public com.ywt.gapi.system.SaveMiniProgramLogResponse saveMiniProgramLog(com.ywt.gapi.system.SaveMiniProgramLogRequest request){
            throw unimplementedMethodException(saveMiniProgramLogMethod);
        }

        @Override
        public com.ywt.gapi.system.GetDeptResponse getDept(com.ywt.gapi.system.GetDeptRequest request){
            throw unimplementedMethodException(getDeptMethod);
        }

        @Override
        public com.ywt.gapi.system.CloudPlatformHospitalListResponse getCloudPlatformHospitalList(com.ywt.gapi.system.HospitalListRequest request){
            throw unimplementedMethodException(getCloudPlatformHospitalListMethod);
        }

        @Override
        public com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeResponse getSystemDicTypeItemListByTypeCode(com.ywt.gapi.system.GetSystemDicTypeItemListByTypeCodeRequest request){
            throw unimplementedMethodException(getSystemDicTypeItemListByTypeCodeMethod);
        }

        @Override
        public com.ywt.gapi.system.InvalidateCacheResponse invalidateCache(com.ywt.gapi.system.InvalidateCacheRequest request){
            throw unimplementedMethodException(invalidateCacheMethod);
        }

        @Override
        public com.ywt.gapi.system.GetCacheKeysResponse getCacheKeys(com.ywt.gapi.system.GetCacheKeysRequest request){
            throw unimplementedMethodException(getCacheKeysMethod);
        }

        @Override
        public com.ywt.gapi.system.GetConfigByTypeResponse getConfigByType(com.ywt.gapi.system.GetConfigByTypeRequest request){
            throw unimplementedMethodException(getConfigByTypeMethod);
        }

        @Override
        public com.ywt.gapi.system.GetHospitalByHospitalIdsResponse getHospitalByHospitalIds(com.ywt.gapi.system.GetHospitalByHospitalIdsRequest request){
            throw unimplementedMethodException(getHospitalByHospitalIdsMethod);
        }

        @Override
        public com.ywt.gapi.system.SaveGPSRecordResponse saveGPSRecord(com.ywt.gapi.system.SaveGPSRecordRequest request){
            throw unimplementedMethodException(saveGPSRecordMethod);
        }

        @Override
        public com.ywt.gapi.system.GetConfigByIdResponse getConfigById(com.ywt.gapi.system.GetConfigByIdRequest request){
            throw unimplementedMethodException(getConfigByIdMethod);
        }

        @Override
        public com.ywt.gapi.system.SearchDeptListByNameResponse searchDeptListByName(com.ywt.gapi.system.SearchDeptListByNameRequest request){
            throw unimplementedMethodException(searchDeptListByNameMethod);
        }

        @Override
        public com.ywt.gapi.system.SearchDoctorListByNameResponse searchDoctorListByName(com.ywt.gapi.system.SearchDoctorListByNameRequest request){
            throw unimplementedMethodException(searchDoctorListByNameMethod);
        }

        @Override
        public com.ywt.gapi.system.SearchDoctorListByNameResponse searchDoctorByNameFromMultiHosp(com.ywt.gapi.system.SearchDoctorByNameFromMultiHospRequest request){
            throw unimplementedMethodException(searchDoctorByNameFromMultiHospMethod);
        }

        @Override
        public com.ywt.gapi.system.RetRegisteredAnnouncementResponse getRegisteredAnnouncement(com.ywt.gapi.system.RetRegisteredAnnouncementRequest request){
            throw unimplementedMethodException(getRegisteredAnnouncementMethod);
        }

        @Override
        public com.ywt.gapi.Result checkHasAmbulanceRight(com.ywt.gapi.system.CheckHasAmbulanceRightRequest request){
            throw unimplementedMethodException(checkHasAmbulanceRightMethod);
        }

        @Override
        public com.ywt.gapi.Result saveAmbulanceLocation(com.ywt.gapi.system.SaveAmbulanceLocationRequest request){
            throw unimplementedMethodException(saveAmbulanceLocationMethod);
        }

        @Override
        public com.ywt.gapi.system.CommonAnnouncementResponse showCommonAnnouncement(com.ywt.gapi.system.CommonAnnouncementRequest request){
            throw unimplementedMethodException(showCommonAnnouncementMethod);
        }

        @Override
        public com.ywt.gapi.system.GuideListResponse getGuideList(com.ywt.gapi.system.GuideListRequest request){
            throw unimplementedMethodException(getGuideListMethod);
        }

        @Override
        public com.ywt.gapi.system.AreaHospitalListResponse getHospitalListByType(com.ywt.gapi.system.GetHospitalListByTypeRequest request){
            throw unimplementedMethodException(getHospitalListByTypeMethod);
        }

        @Override
        public com.ywt.gapi.system.IntroduceResponse getIntroduce(com.ywt.gapi.system.IntroduceRequest request){
            throw unimplementedMethodException(getIntroduceMethod);
        }

        @Override
        public com.ywt.gapi.system.HospitalMedicalDeptListResponse hospitalMedicalDeptList(com.ywt.gapi.system.HospitalMedicalDeptListRequest request){
            throw unimplementedMethodException(hospitalMedicalDeptListMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
