/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.system;

import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.PathResolver;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.ServerService;
import org.apache.dubbo.rpc.TriRpcStatus;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.BiStreamMethodHandler;
import org.apache.dubbo.rpc.stub.ServerStreamMethodHandler;
import org.apache.dubbo.rpc.stub.StubInvocationUtil;
import org.apache.dubbo.rpc.stub.StubInvoker;
import org.apache.dubbo.rpc.stub.StubMethodHandler;
import org.apache.dubbo.rpc.stub.StubSuppliers;
import org.apache.dubbo.rpc.stub.UnaryStubMethodHandler;

import com.google.protobuf.Message;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.concurrent.CompletableFuture;

public final class DubboSystemServiceTriple {

    public static final String SERVICE_NAME = SystemService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,SystemService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,SystemProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboSystemServiceTriple::newStub);
        StubSuppliers.addSupplier(SystemService.JAVA_SERVICE_NAME,  DubboSystemServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(SystemService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static SystemService newStub(Invoker<?> invoker) {
        return new SystemServiceStub((Invoker<SystemService>)invoker);
    }

    private static final StubMethodDescriptor getHospitalDeptListMethod = new StubMethodDescriptor("getHospitalDeptList",
    HospitalDeptListRequest.class, com.ywt.gapi.system.HospitalDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), HospitalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalDeptListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalDeptListAsyncMethod = new StubMethodDescriptor("getHospitalDeptList",
    HospitalDeptListRequest.class, CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), HospitalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalDeptListResponse::parseFrom);

    private static final StubMethodDescriptor getHospitalDeptListProxyAsyncMethod = new StubMethodDescriptor("getHospitalDeptListAsync",
    HospitalDeptListRequest.class, com.ywt.gapi.system.HospitalDeptListResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), HospitalDeptListRequest::parseFrom,
    com.ywt.gapi.system.HospitalDeptListResponse::parseFrom);





    public static class SystemServiceStub implements SystemService{
        private final Invoker<SystemService> invoker;

        public SystemServiceStub(Invoker<SystemService> invoker) {
            this.invoker = invoker;
        }

        @Override
        public com.ywt.gapi.system.HospitalDeptListResponse getHospitalDeptList(HospitalDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalDeptListMethod, request);
        }

        public CompletableFuture<com.ywt.gapi.system.HospitalDeptListResponse> getHospitalDeptListAsync(HospitalDeptListRequest request){
            return StubInvocationUtil.unaryCall(invoker, getHospitalDeptListAsyncMethod, request);
        }

        @Override
        public void getHospitalDeptList(HospitalDeptListRequest request, StreamObserver<com.ywt.gapi.system.HospitalDeptListResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, getHospitalDeptListMethod , request, responseObserver);
        }



    }

    public static abstract class SystemServiceImplBase implements SystemService, ServerService<SystemService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<SystemService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalDeptList" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/getHospitalDeptListAsync" );

            BiConsumer<HospitalDeptListRequest, StreamObserver<com.ywt.gapi.system.HospitalDeptListResponse>> getHospitalDeptListFunc = this::getHospitalDeptList;
            handlers.put(getHospitalDeptListMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalDeptListFunc));
            BiConsumer<HospitalDeptListRequest, StreamObserver<com.ywt.gapi.system.HospitalDeptListResponse>> getHospitalDeptListAsyncFunc = syncToAsync(this::getHospitalDeptList);
            handlers.put(getHospitalDeptListProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(getHospitalDeptListAsyncFunc));




            return new StubInvoker<>(this, url, SystemService.class, handlers);
        }


        @Override
        public com.ywt.gapi.system.HospitalDeptListResponse getHospitalDeptList(HospitalDeptListRequest request){
            throw unimplementedMethodException(getHospitalDeptListMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
