/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.base.idCard;

import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.PathResolver;
import org.apache.dubbo.rpc.RpcException;
import org.apache.dubbo.rpc.ServerService;
import org.apache.dubbo.rpc.TriRpcStatus;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.BiStreamMethodHandler;
import org.apache.dubbo.rpc.stub.ServerStreamMethodHandler;
import org.apache.dubbo.rpc.stub.StubInvocationUtil;
import org.apache.dubbo.rpc.stub.StubInvoker;
import org.apache.dubbo.rpc.stub.StubMethodHandler;
import org.apache.dubbo.rpc.stub.StubSuppliers;
import org.apache.dubbo.rpc.stub.UnaryStubMethodHandler;

import com.google.protobuf.Message;

import java.util.HashMap;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.concurrent.CompletableFuture;

public final class DubboIdCardServiceTriple {

    public static final String SERVICE_NAME = IdCardService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,IdCardService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,BaseIdCardProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboIdCardServiceTriple::newStub);
        StubSuppliers.addSupplier(IdCardService.JAVA_SERVICE_NAME,  DubboIdCardServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(IdCardService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static IdCardService newStub(Invoker<?> invoker) {
        return new IdCardServiceStub((Invoker<IdCardService>)invoker);
    }

    private static final StubMethodDescriptor validateIdCardMethod = new StubMethodDescriptor("validateIdCard",
    ValidateIdCardRequest.class, ValidateIdCardResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), ValidateIdCardRequest::parseFrom,
    ValidateIdCardResponse::parseFrom);

    private static final StubMethodDescriptor validateIdCardAsyncMethod = new StubMethodDescriptor("validateIdCard",
    ValidateIdCardRequest.class, CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), ValidateIdCardRequest::parseFrom,
    ValidateIdCardResponse::parseFrom);

    private static final StubMethodDescriptor validateIdCardProxyAsyncMethod = new StubMethodDescriptor("validateIdCardAsync",
    ValidateIdCardRequest.class, ValidateIdCardResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), ValidateIdCardRequest::parseFrom,
    ValidateIdCardResponse::parseFrom);





    public static class IdCardServiceStub implements IdCardService{
        private final Invoker<IdCardService> invoker;

        public IdCardServiceStub(Invoker<IdCardService> invoker) {
            this.invoker = invoker;
        }

        @Override
        public ValidateIdCardResponse validateIdCard(ValidateIdCardRequest request){
            return StubInvocationUtil.unaryCall(invoker, validateIdCardMethod, request);
        }

        public CompletableFuture<ValidateIdCardResponse> validateIdCardAsync(ValidateIdCardRequest request){
            return StubInvocationUtil.unaryCall(invoker, validateIdCardAsyncMethod, request);
        }

        @Override
        public void validateIdCard(ValidateIdCardRequest request, StreamObserver<ValidateIdCardResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, validateIdCardMethod , request, responseObserver);
        }



    }

    public static abstract class IdCardServiceImplBase implements IdCardService, ServerService<IdCardService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<IdCardService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/validateIdCard" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/validateIdCardAsync" );

            BiConsumer<ValidateIdCardRequest, StreamObserver<ValidateIdCardResponse>> validateIdCardFunc = this::validateIdCard;
            handlers.put(validateIdCardMethod.getMethodName(), new UnaryStubMethodHandler<>(validateIdCardFunc));
            BiConsumer<ValidateIdCardRequest, StreamObserver<ValidateIdCardResponse>> validateIdCardAsyncFunc = syncToAsync(this::validateIdCard);
            handlers.put(validateIdCardProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(validateIdCardAsyncFunc));




            return new StubInvoker<>(this, url, IdCardService.class, handlers);
        }


        @Override
        public ValidateIdCardResponse validateIdCard(ValidateIdCardRequest request){
            throw unimplementedMethodException(validateIdCardMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
