package com.ywt.outpatient.taihe.rpc.domain.enums;

/**
 * 挂号订单状态
 *
 * @author johnson lin
 * @date 2019/4/30 6:43 PM
 */
public enum RegisteredStatus {
    /**
     * 预约成功
     */
    REGISTER("已预约", 1),
    /**
     * 用户取消预约
     */
    CANCEL("已取消", 1 << 1),
    /**
     * 已取号
     * 南方医院号源的 取号 意思是 成功支付挂号费；
     * 太和分院号源的 取号 意思是 报到。
     */
    TOOK("已取号", 1 << 2),
    /**
     * 挂号未就诊（未用到，保留）
     * 系统目前无法判断用户是否就诊
     */
    EXPIRED("已过期", 1 << 3),
    /**
     * 已退号
     * 南方医院号源的 退号 意思是 已退还挂号费；
     * 太和分院号源的 退号 意思是 已取消。
     */
    BACK("已退号", 1 << 4);

    RegisteredStatus(String displayName, int value) {
        this.displayName = displayName;
        this.value = value;
    }

    private final String displayName;

    private final int value;

    public String getDisplayName() {
        return displayName;
    }

    public int getValue() {
        return value;
    }
}
